﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 入力用ツール群
    /// </summary>
    public abstract class YCApplicationInputTools
    {
        /// <summary>
        /// 顧客グループ選択
        /// <para>選択なしは許可しません。</para>
        /// </summary>
        /// <param name="code">選択された顧客グループコード（出力用）</param>
        /// <param name="name">選択された顧客グループ名（出力用）</param>
        /// <returns>OKならtrue、Cancelならfalse</returns>
        public static bool SelectCustomerGroupCode(out string code, out string name)
        {
            return YCApplicationInputTools.SelectCustomerGroupCode(out code, out name, false);
        }

        /// <summary>
        /// 顧客グループ選択
        /// </summary>
        /// <param name="code">選択された顧客グループコード（出力用）</param>
        /// <param name="name">選択された顧客グループ名（出力用）</param>
        /// <param name="allowSelectNothing">選択なしを許可するならtrue</param>
        /// <returns>OKならtrue、Cancelならfalse</returns>
        public static bool SelectCustomerGroupCode(out string code, out string name, bool allowSelectNothing)
        {
            bool ans = false;
            code = string.Empty;
            name = string.Empty;

            dlgCustomerGroupTree dlg = null;

            try
            {
                dlg = new dlgCustomerGroupTree();
                dlg.AllowSelectNothing = allowSelectNothing;

                DialogResult ret = dlg.ShowDialog(YCApplication.Instance.MdiParent);
                if (ret == DialogResult.OK)
                {
                    code = dlg.SelectedCustomerGroupCode;
                    name = dlg.SelectedCustomerGroupName;

                    ans = true;
                }

                dlg.Dispose();
                dlg = null;

                return ans;
            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }

                code = string.Empty;
                name = string.Empty;
                throw e;
            }
        }

    }
}
