﻿using System;
using System.Collections.Generic;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 自動採番マネージャクラス
    /// Singletonパターンです。
    /// GetAutoNoメソッドを使って、「Ｍ採番」テーブルより
    /// オブジェクトコードを指定して番号を取得します。
    /// </summary>
    public class YCAutoNoManager : YCAutoNoBaseForSQLServer
    {
        private static YCAutoNoManager _instance = null;

        private YCAutoNoManager()
            : base()
        {
        }

        public override void Dispose()
        {
            if (this.IsDisposable)
            {
                _instance = null;
            }
            base.Dispose();
        }

        //Singletonのインスタンスです。
        public static YCAutoNoManager Instance
        {
            get
            {
                if (_instance == null) _instance = new YCAutoNoManager();
                return _instance;
            }
        }

        /// <summary>
        /// 顧客コードを取得します。
        /// </summary>
        /// <returns>顧客コード</returns>
        public string GetCustomerCode()
        {
            long code = YCAutoNoManager.Instance.GetAutoNo(YCObjectType.Customer);
            return YCConvert.ToString(code).Trim().PadLeft(YCDbColumnLength.CustomerCode, '0');
        }

        /// <summary>
        /// 顧客グループコードを取得します。
        /// </summary>
        /// <returns>顧客グループコード</returns>
        public string GetCustomerGroupCode()
        {
            long code = YCAutoNoManager.Instance.GetAutoNo(YCObjectType.CustomerGroup);
            return YCConvert.ToString(code).Trim().PadLeft(YCDbColumnLength.CustomerGroupCode, '0');
        }
    }
}
