﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客オブジェクト
    /// </summary>
    public class YCCustomer : YCDbObject
    {
        private YCCustomerTelephoneList _telList = null;
        private YCCustomerFaxList _faxList = null;
        private YCCustomerMailList _mailList = null;
        private YCCustomerBelongingGroupList _groupList = null;

        public YCCustomer(YCConnection connection)
            : base(connection)
        {
        }

        public override void Dispose()
        {
            if (this.IsDisposable)
            {
                if (_telList != null)
                {
                    _telList.DecrementRef();
                    try { _telList.Dispose(); }
                    catch { }
                    _telList = null;
                }

                if (_faxList != null)
                {
                    _faxList.DecrementRef();
                    try { _faxList.Dispose(); }
                    catch { }
                    _faxList = null;
                }

                if (_mailList != null)
                {
                    _mailList.DecrementRef();
                    try { _mailList.Dispose(); }
                    catch { }
                    _mailList = null;
                }

                if (_groupList != null)
                {
                    _groupList.DecrementRef();
                    try { _groupList.Dispose(); }
                    catch { }
                    _groupList = null;
                }

            }

            base.Dispose();
        }

        /// <summary>
        /// テーブル名
        /// </summary>
        public override string TableName
        {
            get
            {
                return "Ｍ顧客";
            }
        }

        /// <summary>
        /// 主キー
        /// </summary>
        public override string KeyColumnName
        {
            get
            {
                return "顧客コード";
            }
        }

        /// <summary>
        /// 主キーの型
        /// </summary>
        protected override DbType KeyColumnDataType
        {
            get
            {
                return DbType.AnsiString;
            }
        }

        /// <summary>
        /// 主キーの桁数
        /// </summary>
        protected override int KeyColumnLength
        {
            get
            {
                return YCDbColumnLength.CustomerCode;
            }
        }

        //キー項目（ユーザＩＤ）のデータは自動採番しない。
        public override object GetNewKeyCode()
        {
            return YCAutoNoManager.Instance.GetCustomerCode();
        }

        /// <summary>
        /// 新規行の初期化
        /// </summary>
        /// <param name="newRow">新規行</param>
        protected override void InitializeNewRow(System.Data.DataRow newRow)
        {
            newRow["法個人コード"] = YCCustomerStatus.Person;
            newRow["法人格配置コード"] = YCCorporateStatusPosition.Non;
            newRow["無効フラグ"] = 0;
        }

        /// <summary>
        /// 
        /// </summary>
        public override bool HasChanges
        {
            get
            {
                if (_telList != null)
                {
                    if (_telList.HasChanges) return true;
                }

                if (_faxList != null)
                {
                    if (_faxList.HasChanges) return true;
                }

                if (_mailList != null)
                {
                    if (_mailList.HasChanges) return true;
                }

                if (_groupList != null)
                {
                    if (_groupList.HasChanges) return true;
                }

                return base.HasChanges;
            }
        }

        /// <summary>
        /// エントリーチェック
        /// <para>外部から呼び出します。保存時に自動的には呼び出しません。</para>
        /// </summary>
        /// <param name="errorString">エラーメッセージ格納用文字列</param>
        /// <returns>エラーがあればtrue</returns>
        public override bool EntryCheck(out string errorString)
        {
            //新規ユーザＩＤの重複チェック
            try
            {
                errorString = string.Empty;
                bool ans = true;

                if (YCConvert.ToString(this["顧客名"]) == "")
                {
                    errorString += "顧客名が未入力です。";
                    ans = false;
                }

                if (YCConvert.ToString(this["顧客名カナ"]) == "")
                {
                    if (errorString != "") errorString += "\n";
                    errorString += "顧客名カナが未入力です。";
                    ans = false;
                }

                return ans;
            }
            catch (Exception e)
            {
                errorString = "ユーザＩＤの確認に失敗しました。\n" + e.Message;
                return false;
            }
        }

        /// <summary>
        /// 顧客の削除
        /// </summary>
        public override void Delete()
        {
            //新規追加の場合は何もしない。
            if (this.IsNew) return;
            if (this.KeyCode == "") throw new YCException("顧客コードが不正です。");
 
            try
            {
                this.Connection.BeginTrans();

                //顧客電話
                string sql = "DELETE FROM Ｍ顧客電話 WHERE 顧客コード = " + YCDbHelper.SqlString(this.KeyCode) + ";";
                this.Connection.Execute(sql);

                //顧客ＦＡＸ
                sql = "DELETE FROM Ｍ顧客ＦＡＸ WHERE 顧客コード = " + YCDbHelper.SqlString(this.KeyCode) + ";";
                this.Connection.Execute(sql);

                //顧客メール
                sql = "DELETE FROM Ｍ顧客メール WHERE 顧客コード = " + YCDbHelper.SqlString(this.KeyCode) + ";";
                this.Connection.Execute(sql);

                //顧客グループマップ
                sql = "DELETE FROM Ｄ顧客グループマップ WHERE 顧客コード = " + YCDbHelper.SqlString(this.KeyCode) + ";";
                this.Connection.Execute(sql);

                //顧客
                sql = "DELETE FROM Ｍ顧客 WHERE 顧客コード = " + YCDbHelper.SqlString(this.KeyCode) + ";";
                this.Connection.Execute(sql);

                this.Connection.Commit();
            }
            catch (Exception e)
            {
                this.Connection.Rollback();

                throw e;
            }
        }

        #region 保存処理

        /// <summary>
        /// 保存の前処理です。
        /// 先にサブリストを保存します。
        /// </summary>
        /// <returns>保存をキャンセルするならtrue</returns>
        protected override bool PreSave()
        {
            this.Connection.BeginTrans();
            this.SetTrans();    //トランザクションをデータベースオブジェクトに設定

            if (_telList != null) _telList.Save();
            if (_faxList != null) _faxList.Save();
            if (_mailList != null) _mailList.Save();
            if (_groupList != null) _groupList.Save();

            return base.PreSave();
        }

        /// <summary>
        /// 
        /// </summary>
        protected override void PostSave()
        {
            base.PostSave();

            this.Connection.Commit();
        }

        protected override void ErrorSave()
        {
            this.Connection.Rollback();

            base.ErrorSave();
        }

        #endregion

        #region プロパティ

        /// <summary>
        /// 住所
        /// </summary>
        public string Address
        {
            get
            {
                string ans = string.Empty;

                if (YCConvert.ToString(this["都道府県コード"]) != "")
                {
                    ans += YCConvert.ToString(YCListManager.Instance.Lists["都道府県"][YCConvert.ToString(this["都道府県コード"])].Value);
                }
                ans += YCConvert.ToString(this["市区町村"]);
                ans += YCConvert.ToString(this["番地"]);
                ans += " " + YCConvert.ToString(this["ビル等"]);

                return ans;
            }
        }

        /// <summary>
        /// 法個人名
        /// </summary>
        public string CustomerStatusName
        {
            get
            {
                string code = YCConvert.ToString(this["法個人コード"]);
                if (code == "")
                {
                    return string.Empty;
                }
                else
                {
                    return YCConvert.ToString(YCListManager.Instance.Lists["法個人"][code].Value);
                }
            }
        }

        /// <summary>
        /// 法人フラグ
        /// </summary>
        public int CompanyFlag
        {
            get
            {
                string code = YCConvert.ToString(this["法個人コード"]);
                if (code == "")
                {
                    return 0;
                }
                else
                {
                    return YCConvert.ToInt(YCListManager.Instance.Lists["法個人"][code].GetItemValue("法人フラグ"));
                }
            }
        }

        /// <summary>
        /// 電話リスト
        /// </summary>
        public YCCustomerTelephoneList TelephoneList
        {
            get
            {
                //新規の場合は顧客コードが確定されていないのでエラー
                if (this.IsNew) throw new YCException("新規顧客オブジェクトが確定されていません。");

                if (_telList == null)
                {
                    _telList = new YCCustomerTelephoneList(this.Connection);
                    _telList.Parent = this;
                    _telList.IncrementRef();
                    _telList.Fill();
                }

                return _telList;
            }
        }

        /// <summary>
        /// ＦＡＸリスト
        /// </summary>
        public YCCustomerFaxList FaxList
        {
            get
            {
                //新規の場合は顧客コードが確定されていないのでエラー
                if (this.IsNew) throw new YCException("新規顧客オブジェクトが確定されていません。");

                if (_faxList == null)
                {
                    _faxList = new YCCustomerFaxList(this.Connection);
                    _faxList.Parent = this;
                    _faxList.IncrementRef();
                    _faxList.Fill();
                }

                return _faxList;
            }
        }

        /// <summary>
        /// メールリスト
        /// </summary>
        public YCCustomerMailList MailList
        {
            get
            {
                //新規の場合は顧客コードが確定されていないのでエラー
                if (this.IsNew) throw new YCException("新規顧客オブジェクトが確定されていません。");

                if (_mailList == null)
                {
                    _mailList = new YCCustomerMailList(this.Connection);
                    _mailList.Parent = this;
                    _mailList.IncrementRef();
                    _mailList.Fill();
                }

                return _mailList;
            }
        }

        /// <summary>
        /// グループリスト
        /// </summary>
        public YCCustomerBelongingGroupList GroupList
        {
            get
            {
                //新規の場合は顧客コードが確定されていないのでエラー
                if (this.IsNew) throw new YCException("新規顧客オブジェクトが確定されていません。");

                if (_groupList == null)
                {
                    _groupList = new YCCustomerBelongingGroupList(this.Connection);
                    _groupList.Parent = this;
                    _groupList.IncrementRef();
                    _groupList.Fill();
                }

                return _groupList;
            }
        }

        #endregion

    }
}
