﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客所属グループリスト
    /// </summary>
    public class YCCustomerBelongingGroupList : YCCustomerSubList
    {
        public YCCustomerBelongingGroupList(YCConnection connection)
            : base(connection)
        {
        }

        /// <summary>
        /// SELECT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>SELECT文</returns>
        protected override string MakeSQLSelect(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "SELECT Ｄ顧客グループマップ.ＳＥＱＮｏ";
            sql += ", Ｄ顧客グループマップ.顧客グループコード, Ｍ顧客グループ.顧客グループ名";
            sql += " FROM Ｄ顧客グループマップ LEFT JOIN Ｍ顧客グループ ON Ｄ顧客グループマップ.顧客グループコード = Ｍ顧客グループ.顧客グループコード";
            sql += " WHERE Ｄ顧客グループマップ.顧客コード = @顧客コード";
            sql += " ORDER BY Ｍ顧客グループ.顧客グループ名;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;

            return sql;
        }

        /// <summary>
        /// INSERT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>INSERT文</returns>
        protected override string MakeSQLInsert(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "INSERT INTO Ｄ顧客グループマップ (ＳＥＱＮｏ";
            sql += ", 顧客コード, 顧客グループコード";
            sql += ") VALUES (@ＳＥＱＮｏ";
            sql += ", @顧客コード, @顧客グループコード);";

            YCDbHelper.AddParameter(this.Connection, parameters, "@ＳＥＱＮｏ", DbType.Guid).SourceColumn = "ＳＥＱＮｏ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客グループコード", DbType.AnsiString, YCDbColumnLength.CustomerGroupCode).SourceColumn = "顧客グループコード";

            return sql;
        }

        /// <summary>
        /// DELETE文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>DELETE文</returns>
        protected override string MakeSQLDelete(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "DELETE FROM Ｄ顧客グループマップ WHERE ＳＥＱＮｏ = @ＳＥＱＮｏ;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@ＳＥＱＮｏ", DbType.Guid).SourceColumn = "ＳＥＱＮｏ";

            return sql;
        }

        public void Add(string code, string name)
        {
            DataRow row = null;

            if (this.StringDataIsExisting("顧客グループコード", code)) throw new YCException("既に登録されています。");

            try
            {
                row = this.NewRow;
                row["ＳＥＱＮｏ"] = YCAutoNoManager.Instance.GetGuid();
                row["顧客グループコード"] = code;
                row["顧客グループ名"] = name;

                this.Table.Rows.Add(row);
                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// 削除
        /// </summary>
        /// <param name="code">ＳＥＱＮｏ</param>
        public override void Delete(Guid code)
        {
            if (code == Guid.Empty) throw new YCException("ＳＥＱＮｏが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["ＳＥＱＮｏ"]) == code)
                {
                    row.Delete();
                    break;
                }
            }
        }
    }
}
