﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客グループ登録用のリスト
    /// </summary>
    public class YCCustomerGroupEntryList : YCDbList
    {
        private string _parentCode = string.Empty;

        public YCCustomerGroupEntryList(YCConnection connection)
            : base(connection)
        {
        }

        //保存可
        public override bool AllowSave
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// 所属顧客グループコードを取得・設定します。
        /// <para>上位の顧客グループコード</para>
        /// </summary>
        public string ParentCode
        {
            get
            {
                return _parentCode;
            }
            set
            {
                _parentCode = value;
            }
        }

        /// <summary>
        /// 所属顧客グループ名を取得します。
        /// <para>上位の顧客グループ名</para>
        /// </summary>
        public string ParentName
        {
            get
            {
                return YCConvert.ToString(this.Connection.DLookup("顧客グループ名", "Ｍ顧客グループ", "顧客グループコード = " + YCDbHelper.SqlString(this.ParentCode)));
            }
        }

        /// <summary>
        /// データベースのレコードに所属顧客グループがあればtrue
        /// </summary>
        public bool ParentIsExists
        {
            get
            {
                if (this.ParentIsRoot)
                {
                    //ルートは必ずある。
                    return true;
                }
                else if (this.Connection.DCount("顧客グループコード", "Ｍ顧客グループ", "顧客グループコード = " + YCDbHelper.SqlString(this.ParentCode)) > 0)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// ルート直下の顧客グループはtrue
        /// </summary>
        public bool ParentIsRoot
        {
            get
            {
                if (this.ParentCode == "") return true;
                else return false;
            }
        }

        protected override string MakeSQLSelect(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "SELECT Ｍ顧客グループ.顧客グループコード, Ｍ顧客グループ.所属顧客グループコード";
            sql += ", Ｍ顧客グループ.顧客グループ名, Ｍ顧客グループ.プライオリティ";
            sql += " FROM Ｍ顧客グループ";

            sql += " WHERE ";
            if (this.ParentCode == "")
            {
                sql += "(Ｍ顧客グループ.所属顧客グループコード Is Null OR Ｍ顧客グループ.所属顧客グループコード = '')";
            }
            else
            {
                sql += "Ｍ顧客グループ.所属顧客グループコード = @所属顧客グループコード";
            }

            sql += " ORDER BY Ｍ顧客グループ.プライオリティ;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@所属顧客グループコード", DbType.AnsiString, YCDbColumnLength.CustomerGroupCode).Value = YCConvert.ToString(this.ParentCode);

            return sql;
        }

        /// <summary>
        /// 新規顧客グループの追加
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public void Add(YCDataContainer container)
        {
            if (container == null) return;
            this.Add(YCConvert.ToString(container["顧客グループ名"]), YCConvert.ToInt(container["プライオリティ"]));
        }

        /// <summary>
        /// 新規顧客グループの追加
        /// </summary>
        /// <param name="groupName">顧客グループ名</param>
        /// <param name="priority">プライオリティ</param>
        public void Add(string groupName, int priority)
        {
            DataRow row = null;

            if (!this.ParentIsExists) throw new YCException("上位の顧客グループが不正です。");

            try
            {
                row = this.NewRow;
                row["顧客グループコード"] = YCAutoNoManager.Instance.GetCustomerGroupCode();

                if (this.ParentCode != "")
                {
                    row["所属顧客グループコード"] = this.ParentCode;
                }

                row["顧客グループ名"] = groupName;
                row["プライオリティ"] = priority;

                this.Table.Rows.Add(row);

                row = null;

                //追加後、すぐに保存
                this.Save();
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// 顧客グループの変更
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public void Update(YCDataContainer container)
        {
            if (container == null) return;
            this.Update(YCConvert.ToString(container["顧客グループコード"]), YCConvert.ToString(container["顧客グループ名"]), YCConvert.ToInt(container["プライオリティ"]));
        }

        /// <summary>
        /// 顧客グループの変更
        /// </summary>
        /// <param name="groupCode">顧客グループコード</param>
        /// <param name="groupName">顧客グループ名</param>
        /// <param name="priority">プライオリティ</param>
        public void Update(string groupCode, string groupName, int priority)
        {
            if (groupCode == "") return;

            int index = this.StringDataToIndex("顧客グループコード", groupCode);
            if (index < 0) throw new YCException("顧客グループコードが不正です。");

            this.Table.Rows[index]["顧客グループ名"] = groupName;
            this.Table.Rows[index]["プライオリティ"] = priority;

            //変更後、すぐに保存
            this.Save();
        }

        /// <summary>
        /// 指定のコードの顧客グループを削除します。
        /// <para>下位階層に顧客グループがある場合は削除不可</para>
        /// </summary>
        /// <param name="targetCode"></param>
        public void Delete(string targetCode)
        {
            int index = this.StringDataToIndex("顧客グループコード", targetCode);
            if (index < 0) throw new YCException("顧客グループコードが不正です。");

            if (this.Connection.DCount("顧客グループコード", "Ｍ顧客グループ", "所属顧客グループコード = " + YCDbHelper.SqlString(targetCode)) != 0)
            {
                throw new YCException("下位階層のある顧客グループは削除できません。");
            }

            //すぐに保存
            try
            {
                //トランザクションの開始
                this.BeginTrans();

                //登録されている顧客を顧客グループから切り離す。
                string sql = "DELETE FROM Ｄ顧客グループマップ WHERE 顧客グループコード = " + YCDbHelper.SqlString(targetCode);
                this.Connection.Execute(sql);

                this.Table.Rows[index].Delete();
                this.Save();

                //トランザクションのコミット
                this.Commit();
            }
            catch (Exception e)
            {
                this.Connection.Rollback();
                throw e;
            }


            
        }
    }
}
