﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

using YSystem;
using YSystem.Net.Mail;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客グループメール送信用クラス
    /// </summary>
    public class YCCustomerGroupMail : YCRefCountObject
    {
        private YCConnection _connection = null;

        private string _accountCode = string.Empty;
        private string _customerGroupCode = string.Empty;
        private bool _includeChildrenCustomerGroup = false;
        private string _mailTypeCode = YCMailType.PC;
        private string _mailSubject = string.Empty;
        private string _mailMessage = string.Empty;

        public YCCustomerGroupMail(YCConnection connection)
            : base()
        {
            if (connection == null) throw new YCException("データベース接続オブジェクトが不正です。");
            _connection = connection;
        }

        public override void Dispose()
        {
            if (this.IsDisposable)
            {
                _connection = null;
            }

            base.Dispose();
        }

        #region プロパティ

        /// <summary>
        /// データベース接続オブジェクトの取得
        /// </summary>
        private YCConnection Connection
        {
            get
            {
                return _connection;
            }
        }

        /// <summary>
        /// 送信元メールアカウントコード
        /// </summary>
        public string AccountCode
        {
            get
            {
                return _accountCode;
            }
            set
            {
                _accountCode = value;
            }
        }

        /// <summary>
        /// 送信先顧客グループコード
        /// </summary>
        public string CustomerGroupCode
        {
            get
            {
                return _customerGroupCode;
            }
            set
            {
                _customerGroupCode = value;
            }
        }

        /// <summary>
        /// 子顧客グループも含むかどうかを取得、設定します。
        /// <para>デフォルトはfalse</para>
        /// </summary>
        public bool IncludeChildrenCustomerGroup
        {
            get
            {
                return _includeChildrenCustomerGroup;
            }
            set
            {
                _includeChildrenCustomerGroup = value;
            }
        }

        /// <summary>
        /// 送信対象のメール種別コード
        /// <para>デフォルトはPC</para>
        /// </summary>
        public string MailTypeCode
        {
            get
            {
                return _mailTypeCode;
            }
            set
            {
                _mailTypeCode = value;
            }
        }

        /// <summary>
        /// メールの件名
        /// </summary>
        public string MailSubject
        {
            get
            {
                return _mailSubject;
            }
            set
            {
                _mailSubject = value;
            }
        }

        /// <summary>
        /// メールの本文
        /// </summary>
        public string MailMessage
        {
            get
            {
                return _mailMessage;
            }
            set
            {
                _mailMessage = value;
            }
        }

        /// <summary>
        /// 対象メールアドレス取得用SELECT文
        /// </summary>
        private string MakeSQLSelect
        {
            get
            {
                string sql = "SELECT DISTINCT Ｍ顧客.顧客名, Ｍ顧客.顧客名カナ,Ｍ法個人.法人フラグ, Ｍ顧客.法人担当者名";
                sql += ", Ｍ顧客メール.メールアドレス";
                sql += " FROM Ｍ顧客 INNER JOIN Ｍ顧客メール ON Ｍ顧客.顧客コード = Ｍ顧客メール.顧客コード";
                sql += " AND Ｍ顧客メール.メール種別コード = " + YCDbHelper.SqlString(this.MailTypeCode);
                sql += " AND Ｍ顧客メール.グループ一括メールフラグ = -1";
                sql += " INNER JOIN Ｄ顧客グループマップ ON Ｍ顧客.顧客コード = Ｄ顧客グループマップ.顧客コード";
                sql += " LEFT JOIN Ｍ法個人 ON Ｍ顧客.法個人コード = Ｍ法個人.法個人コード";
                sql += " WHERE (Ｍ顧客.無効フラグ = 0 OR Ｍ顧客.無効フラグ IS NULL)";
                
                sql += " AND (Ｄ顧客グループマップ.顧客グループコード = " + YCDbHelper.SqlString(this.CustomerGroupCode);

                //
                //YWorks Software Framework キーが設定されている場合のみ処理可能です。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //
                if (this.IncludeChildrenCustomerGroup && YCLicenseKeys.FrameworkKey != "")
                {
                    string[] children = YCApplicationHelper.GetChildrenGroupCodes(this.CustomerGroupCode, this.Connection);
                    if (children != null)
                    {
                        foreach (string child in children)
                        {
                            sql += " OR Ｄ顧客グループマップ.顧客グループコード = " + YCDbHelper.SqlString(child);
                        }
                    }
                }
                sql += ")";

                sql += " ORDER BY Ｍ顧客.顧客名カナ;";

                return sql;
            }
        }

        #endregion

        #region メソッド

        /// <summary>
        /// メール送信処理
        /// </summary>
        public void Send()
        {
            YCSmtpClient server = null;
            YCKeyValuePair<string, object> account = null;
            YCDataTableReader dtr = null;
            YCMailMessage message = null;
            bool errFlag = false;
            string errMsg = string.Empty;

            if (this.CustomerGroupCode == "") throw new YCException("送信先顧客グループが不正です。");
            if (this.MailTypeCode  == "") throw new YCException("メール種別が不正です。");
            
            if (this.AccountCode == "") throw new YCException("送信元アカウントが不正です。");

            try
            {
                //送信サーバの設定
                account = YCListManager.Instance.Lists["メールアカウント"][this.AccountCode];
                if (account == null) throw new YCException("送信元アカウントが不正です。");

                server = new YCSmtpClient();
                server.SmtpServer = YCConvert.ToString(account.GetItemValue("ＳＭＴＰサーバ"));
                server.SmtpPort = YCConvert.ToInt(account.GetItemValue("ＳＭＴＰポート"));
                server.HostName = Dns.GetHostName();
                if (YCConvert.IntegerToBoolean(account.GetItemValue("ＳＭＴＰ認証フラグ")))
                {
                    //認証メソッドはplainを使用する。
                    server.AuthMethod = YCSmtpAuthMethod.Plain;
                    server.AuthUserName = YCConvert.ToString(account.GetItemValue("ＳＭＴＰ認証アカウントＩＤ"));
                    server.AuthUserPassword = YCConvert.ToString(account.GetItemValue("ＳＭＴＰ認証パスワード"));
                }
                else
                {
                    server.AuthMethod = YCSmtpAuthMethod.Non;
                }

                //送信元情報
                string accountAddress = YCConvert.ToString(account.GetItemValue("メールアドレス"));
                string accountDisplayName = YCConvert.ToString(account.GetItemValue("メールアドレス表示名"));

                account = null;

                //対象メールアドレスの取得
                dtr = this.Connection.CreateDataTableReader(this.MakeSQLSelect);
                dtr.MoveFirst();

                while (!dtr.EOF)
                {

                    string targetAddress = YCConvert.ToString(dtr["メールアドレス"]);
                    string targetName = this.GetCustomerName(YCConvert.ToString(dtr["顧客名"]), YCConvert.IntegerToBoolean(dtr["法人フラグ"]), YCConvert.ToString(dtr["法人担当者名"]));
                   
                    message = new YCMailMessage();

                    //
                    //YWorks Software SMPTライブラリ キーが設定されている場合
                    //dot-atom非準拠の一部の携帯メールアドレスにも対応可能です。
                    //（ただし、SMTPサーバも対応している必要がありますので、
                    //  プロバイダ等のサーバ管理者にお問合せ下さい。）
                    //YCLicenseKeys.SmtpKeyプロパティを実装して下さい。
                    //
                    message.YWorksKey = YCLicenseKeys.SmtpKey;
                    
                    message.SetFrom(accountAddress, accountDisplayName);
                    message.Subject = this.MailSubject;
                    message.Body = targetName + "\r\n\r\n" + this.MailMessage;    // text/plain形式
                    
                    try
                    {
                        message.AddTo(targetAddress, targetName);
                        server.Send(message);
                    }
                    catch
                    {
                        errFlag = true;
                        errMsg += "送信エラー:" + targetAddress + "（" + targetName + "）\n";
                    }

                    message.Dispose();
                    message = null;

                    dtr.MoveNext();
                }

                dtr.Dispose();
                dtr = null;

                server.Dispose();
                server = null;

                if (errFlag) throw new YCException("送信に失敗しました。\n\n" + errMsg);

            }
            catch (Exception e)
            {
                if (server != null)
                {
                    try { server.Dispose(); }
                    catch { }
                    server = null;
                }

                if (dtr != null)
                {
                    try { dtr.Dispose(); }
                    catch { }
                    dtr = null;
                }

                if (message != null)
                {
                    try { message.Dispose(); }
                    catch { }
                    message = null;
                }

                account = null;

                throw e;
            }
        }

        #endregion

        #region ローカルメソッド

        /// <summary>
        /// 本文冒頭で使用する宛先顧客名を取得します。
        /// </summary>
        /// <param name="name">顧客名</param>
        /// <param name="isCompany">法人ならtrue</param>
        /// <param name="chargeName">法人担当者名</param>
        /// <returns>宛先顧客名</returns>
        private string GetCustomerName(string name, bool isCompany, string chargeName)
        {
            string ans = string.Empty;

            if (isCompany)
            {
                if (chargeName == "")
                {
                    ans = name + " 御中";
                }
                else
                {
                    ans = name + " " + chargeName + " 様";
                }
            }
            else
            {
                ans = name + " 様";
            }


            return ans;
        }

        #endregion
    }
}
