﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客リスト
    /// </summary>
    public class YCCustomerList : YCDbList
    {
        private string _customerGroupCode = string.Empty;
        private bool _includeChildrenCustomerGroup = false;

        public YCCustomerList(YCConnection connection)
            : base(connection)
        {
        }

        protected override string MakeSQLSelect(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "SELECT DISTINCT Ｍ顧客.顧客コード, Ｍ顧客.顧客名, Ｍ顧客.顧客名カナ";
            sql += ", Ｍ法個人.法個人名, Ｍ法個人.法人フラグ, Ｍ顧客.法人格配置コード";
            sql += ", Ｍ顧客.法人担当者名, Ｍ顧客.法人担当者役職";
            sql += ", Ｍ顧客.郵便番号";
            sql += ", (ISNULL(Ｍ都道府県.都道府県名, '') + ISNULL(市区町村, '') + ISNULL(番地, '') + ' ' + ISNULL(ビル等, '')) AS 住所";
            sql += ", Ｍ顧客.取引メモ, Ｍ顧客.備考";
            sql += " FROM Ｍ顧客";
            if(this.CustomerGroupCode != "")
            {
                sql += " INNER JOIN Ｄ顧客グループマップ ON Ｍ顧客.顧客コード = Ｄ顧客グループマップ.顧客コード";
            }
            sql += " LEFT JOIN Ｍ都道府県 ON Ｍ顧客.都道府県コード = Ｍ都道府県.都道府県コード";
            sql += " LEFT JOIN Ｍ法個人 ON Ｍ顧客.法個人コード = Ｍ法個人.法個人コード";

            sql += " WHERE (Ｍ顧客.無効フラグ = 0 OR Ｍ顧客.無効フラグ IS NULL)";

            if (this.CustomerGroupCode != "")
            {
                sql += " AND (Ｄ顧客グループマップ.顧客グループコード = @顧客グループコード";
                YCDbHelper.AddParameter(this.Connection, parameters, "@顧客グループコード", DbType.AnsiString, YCDbColumnLength.CustomerGroupCode).Value = this.CustomerGroupCode;

                //
                //YWorks Software Framework キーが設定されている場合のみ処理可能です。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //
                if (this.IncludeChildrenCustomerGroup && YCLicenseKeys.FrameworkKey != "")
                {
                    string[] children = YCApplicationHelper.GetChildrenGroupCodes(this.CustomerGroupCode, this.Connection);
                    if (children != null)
                    {
                        int count = 1;
                        foreach (string child in children)
                        {
                            sql += " OR Ｄ顧客グループマップ.顧客グループコード = @子顧客グループコード" + count.ToString().Trim();
                            YCDbHelper.AddParameter(this.Connection, parameters, "@子顧客グループコード" + count.ToString().Trim(), DbType.AnsiString, YCDbColumnLength.CustomerGroupCode).Value = child;
                            
                            count++;
                        }
                    }
                }
                sql += ")";
            }
            
            sql += " ORDER BY Ｍ顧客.顧客名カナ;";


            return sql;
        }

        public override void RefreshRow(YCDbObject dbObject)
        {
            if (dbObject is YCCustomer)
            {
                int index = this.StringDataToIndex(dbObject.KeyColumnName, dbObject.KeyCode);
                if (index != YCDatabaseConst.NO_INDEX)
                {
                    this.Table.Rows[index]["顧客コード"] = dbObject["顧客コード"];
                    this.Table.Rows[index]["顧客名"] = dbObject["顧客名"];
                    this.Table.Rows[index]["顧客名カナ"] = dbObject["顧客名カナ"];
                    this.Table.Rows[index]["法個人名"] = ((YCCustomer)dbObject).CustomerStatusName;
                    this.Table.Rows[index]["法人フラグ"] = ((YCCustomer)dbObject).CompanyFlag;
                    this.Table.Rows[index]["法人格配置コード"] = dbObject["法人格配置コード"];
                    this.Table.Rows[index]["法人担当者名"] = dbObject["法人担当者名"];
                    this.Table.Rows[index]["法人担当者役職"] = dbObject["法人担当者役職"];
                    this.Table.Rows[index]["郵便番号"] = dbObject["郵便番号"];
                    this.Table.Rows[index]["住所"] = ((YCCustomer)dbObject).Address;
                    this.Table.Rows[index]["取引メモ"] = dbObject["取引メモ"];
                    this.Table.Rows[index]["備考"] = dbObject["備考"];
                }
            }
        }

        public override void  WriteToFile(YCFile file)
        {
            if (!file.IsOpened) throw new YCFileException("ファイルがオープンされていません。");
            if (!this.IsLoaded) throw new YCDbException("データが読込まれていません。");

            //ヘッダの出力
            string line = "\"顧客名\"";
            line += ",\"完全顧客名\"";
            line += ",\"郵便番号\"";
            line += ",\"住所\"";
            line += ",\"取引メモ\"";
            line += ",\"備考\"";

            file.WriteLine(line);

            //データの出力
            this.MoveFirst();
            while (!this.EOF)
            {
                line = "\"" + YCConvert.ToString(this["顧客名"]) + "\"";
                line += ",\"" + this.GetFullCustomerName() + "\"";
                line += ",\"" + YCConvert.ToString(this["郵便番号"]) + "\"";
                line += ",\"" + YCConvert.ToString(this["住所"]) + "\"";
                line += ",\"" + YCConvert.ToString(this["取引メモ"]) + "\"";
                line += ",\"" + YCConvert.ToString(this["備考"]) + "\"";

                file.WriteLine(line);

                this.MoveNext();
            }
        }

        /// <summary>
        /// CSVファイル出力用の完全顧客名です。
        /// <para>カレントレコードを対象とします。</para>
        /// </summary>
        /// <returns>完全顧客名</returns>
        private string GetFullCustomerName()
        {
            string ans = string.Empty;

            if (YCConvert.IntegerToBoolean(this["法人フラグ"]))
            {
                //法人の場合
                if(YCConvert.ToString(this["法人格配置コード"]) == YCCorporateStatusPosition.Ahead)
                {
                    ans = YCConvert.ToString(this["法個人名"]) + " " + YCConvert.ToString(this["顧客名"]);
                }
                else if (YCConvert.ToString(this["法人格配置コード"]) == YCCorporateStatusPosition.Back)
                {
                    ans = YCConvert.ToString(this["顧客名"]) + " " + YCConvert.ToString(this["法個人名"]);
                }
                else
                {
                    ans = YCConvert.ToString(this["顧客名"]);
                }

                if (YCConvert.ToString(this["法人担当者名"]) != "")
                {
                    if (YCConvert.ToString(this["法人担当者役職"]) != "")
                    {
                        ans += " " + YCConvert.ToString(this["法人担当者役職"]);
                    }

                    ans += " " + YCConvert.ToString(this["法人担当者名"]);
                }
            }
            else
            {
                //個人の場合
                ans = YCConvert.ToString(this["顧客名"]);
            }

            return ans;
        }

        #region プロパティ
        
        /// <summary>
        /// 対象顧客グループ
        /// <para>指定無しなら全てを対象とする。</para>
        /// </summary>
        public string CustomerGroupCode
        {
            get
            {
                return _customerGroupCode;
            }
            set
            {
                _customerGroupCode = value;
            }
        }

        /// <summary>
        /// 子顧客グループも含むかどうかを取得、設定します。
        /// <para>デフォルトはfalse</para>
        /// </summary>
        public bool IncludeChildrenCustomerGroup
        {
            get
            {
                return _includeChildrenCustomerGroup;
            }
            set
            {
                _includeChildrenCustomerGroup = value;
            }
        }

        #endregion

    }
}
