﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客メールリスト
    /// </summary>
    public class YCCustomerMailList : YCCustomerSubList
    {
        public YCCustomerMailList(YCConnection connection)
            : base(connection)
        {
        }

        /// <summary>
        /// SELECT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>SELECT文</returns>
        protected override string MakeSQLSelect(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "SELECT Ｍ顧客メール.顧客メールコード";
            sql += ", Ｍ顧客メール.連絡先種別コード, Ｍ連絡先種別.連絡先種別名";
            sql += ", Ｍ顧客メール.メール種別コード, Ｍメール種別.メール種別名";
            sql += ", Ｍ顧客メール.メールアドレス";
            sql += ", Ｍ顧客メール.グループ一括メールフラグ, Ｍ顧客メール.備考";
            sql += " FROM Ｍ顧客メール LEFT JOIN Ｍ連絡先種別 ON Ｍ顧客メール.連絡先種別コード = Ｍ連絡先種別.連絡先種別コード";
            sql += " LEFT JOIN Ｍメール種別 ON Ｍ顧客メール.メール種別コード = Ｍメール種別.メール種別コード";
            sql += " WHERE Ｍ顧客メール.顧客コード = @顧客コード";
            sql += " ORDER BY Ｍ連絡先種別.プライオリティ, Ｍメール種別.プライオリティ;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;

            return sql;
        }

        /// <summary>
        /// INSERT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>INSERT文</returns>
        protected override string MakeSQLInsert(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "INSERT INTO Ｍ顧客メール (顧客メールコード, 顧客コード";
            sql += ", 連絡先種別コード, メール種別コード";
            sql += ", メールアドレス, グループ一括メールフラグ, 備考)";
            sql += " VALUES (@顧客メールコード, @顧客コード";
            sql += ", @連絡先種別コード, @メール種別コード";
            sql += ", @メールアドレス, @グループ一括メールフラグ, @備考);";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客メールコード", DbType.Guid).SourceColumn = "顧客メールコード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;
            YCDbHelper.AddParameter(this.Connection, parameters, "@連絡先種別コード", DbType.AnsiString, YCDbColumnLength.ContactTypeCode).SourceColumn = "連絡先種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@メール種別コード", DbType.AnsiString, YCDbColumnLength.MailTypeCode).SourceColumn = "メール種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@メールアドレス", DbType.AnsiString, YCDbColumnLength.MailAddress).SourceColumn = "メールアドレス";
            YCDbHelper.AddParameter(this.Connection, parameters, "@グループ一括メールフラグ", DbType.Int16).SourceColumn = "グループ一括メールフラグ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@備考", DbType.AnsiString, YCDbColumnLength.Note).SourceColumn = "備考";

            return sql;
        }

        /// <summary>
        /// UPDATE文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>UPDATE文</returns>
        protected override string MakeSQLUpdate(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "UPDATE Ｍ顧客メール SET";
            sql += " 連絡先種別コード = @連絡先種別コード";
            sql += ", メール種別コード = @メール種別コード";
            sql += ", メールアドレス = @メールアドレス";
            sql += ", グループ一括メールフラグ = @グループ一括メールフラグ";
            sql += ", 備考 = @備考";
            sql += " WHERE 顧客メールコード = @顧客メールコード;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@連絡先種別コード", DbType.AnsiString, YCDbColumnLength.ContactTypeCode).SourceColumn = "連絡先種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@メール種別コード", DbType.AnsiString, YCDbColumnLength.MailTypeCode).SourceColumn = "メール種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@メールアドレス", DbType.AnsiString, YCDbColumnLength.MailAddress).SourceColumn = "メールアドレス";
            YCDbHelper.AddParameter(this.Connection, parameters, "@グループ一括メールフラグ", DbType.Int16).SourceColumn = "グループ一括メールフラグ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@備考", DbType.AnsiString, YCDbColumnLength.Note).SourceColumn = "備考";
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客メールコード", DbType.Guid).SourceColumn = "顧客メールコード";

            return sql;
        }

        /// <summary>
        /// DELETE文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>DELETE文</returns>
        protected override string MakeSQLDelete(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "DELETE FROM Ｍ顧客メール WHERE 顧客メールコード = @顧客メールコード;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客メールコード", DbType.Guid).SourceColumn = "顧客メールコード";

            return sql;
        }

        /// <summary>
        /// データコンテナを利用した追加
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public override void Add(YCDataContainer container)
        {
            DataRow row = null;

            try
            {
                row = this.NewRow;
                row["顧客メールコード"] = YCAutoNoManager.Instance.GetGuid();
                row["連絡先種別コード"] = container["連絡先種別コード"];
                if (YCConvert.ToString(row["連絡先種別コード"]) != "")
                {
                    row["連絡先種別名"] = YCListManager.Instance.Lists["連絡先種別"][YCConvert.ToString(row["連絡先種別コード"])].Value;
                }
                row["メール種別コード"] = container["メール種別コード"];
                if (YCConvert.ToString(row["メール種別コード"]) != "")
                {
                    row["メール種別名"] = YCListManager.Instance.Lists["メール種別"][YCConvert.ToString(row["メール種別コード"])].Value;
                }
                row["メールアドレス"] = container["メールアドレス"];
                row["グループ一括メールフラグ"] = container["グループ一括メールフラグ"];
                row["備考"] = container["備考"];

                this.Table.Rows.Add(row);
                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// メールアドレスのみでの追加
        /// </summary>
        /// <param name="data">メールアドレス</param>
        public override void Add(string data)
        {
            DataRow row = null;

            try
            {
                row = this.NewRow;
                row["顧客メールコード"] = YCAutoNoManager.Instance.GetGuid();
                row["メールアドレス"] = data;
                row["メール種別コード"] = YCMailType.PC;
                row["メール種別名"] = YCListManager.Instance.Lists["メール種別"][YCMailType.PC].Value;
                row["グループ一括メールフラグ"] = -1;

                this.Table.Rows.Add(row);
                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// データコンテナを利用した変更
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public override void Update(YCDataContainer container)
        {
            if (YCConvert.ToGuid(container["顧客メールコード"]) == Guid.Empty) throw new YCException("顧客メールコードが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["顧客メールコード"]) == YCConvert.ToGuid(container["顧客メールコード"]))
                {
                    row["連絡先種別コード"] = container["連絡先種別コード"];
                    if (YCConvert.ToString(row["連絡先種別コード"]) != "")
                    {
                        row["連絡先種別名"] = YCListManager.Instance.Lists["連絡先種別"][YCConvert.ToString(row["連絡先種別コード"])].Value;
                    }
                    row["メール種別コード"] = container["メール種別コード"];
                    if (YCConvert.ToString(row["メール種別コード"]) != "")
                    {
                        row["メール種別名"] = YCListManager.Instance.Lists["メール種別"][YCConvert.ToString(row["メール種別コード"])].Value;
                    }
                    row["メールアドレス"] = container["メールアドレス"];
                    row["グループ一括メールフラグ"] = container["グループ一括メールフラグ"];
                    row["備考"] = container["備考"];

                    break;
                }
            }
        }

        /// <summary>
        /// メールアドレスのみの変更
        /// </summary>
        /// <param name="code">顧客メールコード</param>
        /// <param name="data">メールアドレス</param>
        public override void Update(Guid code, string data)
        {
            if (code == Guid.Empty) throw new YCException("顧客メールコードが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["顧客メールコード"]) == code)
                {
                    row["メールアドレス"] = data;
                    break;
                }
            }
        }

        /// <summary>
        /// 削除
        /// </summary>
        /// <param name="code">顧客メールコード</param>
        public override void Delete(Guid code)
        {
            if (code == Guid.Empty) throw new YCException("顧客メールコードが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["顧客メールコード"]) == code)
                {
                    row.Delete();
                    break;
                }
            }
        }
    }
}
