﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客電話リスト
    /// </summary>
    public class YCCustomerTelephoneList : YCCustomerSubList
    {
        public YCCustomerTelephoneList(YCConnection connection)
            : base(connection)
        {
        }

        /// <summary>
        /// SELECT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>SELECT文</returns>
        protected override string MakeSQLSelect(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "SELECT Ｍ顧客電話.顧客電話コード";
            sql += ", Ｍ顧客電話.連絡先種別コード, Ｍ連絡先種別.連絡先種別名";
            sql += ", Ｍ顧客電話.電話種別コード, Ｍ電話種別.電話種別名";
            sql += ", Ｍ顧客電話.電話番号, Ｍ顧客電話.有効時間帯メモ";
            sql += ", Ｍ顧客電話.利用不可フラグ, Ｍ顧客電話.備考";
            sql += " FROM Ｍ顧客電話 LEFT JOIN Ｍ連絡先種別 ON Ｍ顧客電話.連絡先種別コード = Ｍ連絡先種別.連絡先種別コード";
            sql += " LEFT JOIN Ｍ電話種別 ON Ｍ顧客電話.電話種別コード = Ｍ電話種別.電話種別コード";
            sql += " WHERE Ｍ顧客電話.顧客コード = @顧客コード";
            sql += " ORDER BY Ｍ連絡先種別.プライオリティ, Ｍ電話種別.プライオリティ;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;

            return sql;
        }

        /// <summary>
        /// INSERT文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>INSERT文</returns>
        protected override string MakeSQLInsert(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "INSERT INTO Ｍ顧客電話 (顧客電話コード, 顧客コード";
            sql += ", 連絡先種別コード, 電話種別コード";
            sql += ", 電話番号, 有効時間帯メモ";
            sql += ", 利用不可フラグ, 備考)";
            sql += " VALUES (@顧客電話コード, @顧客コード";
            sql += ", @連絡先種別コード, @電話種別コード";
            sql += ", @電話番号, @有効時間帯メモ";
            sql += ", @利用不可フラグ, @備考);";

            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客電話コード", DbType.Guid).SourceColumn = "顧客電話コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客コード", DbType.AnsiString, YCDbColumnLength.CustomerCode).Value = this.CustomerCode;
            YCDbHelper.AddParameter(this.Connection, parameters, "@連絡先種別コード", DbType.AnsiString, YCDbColumnLength.ContactTypeCode).SourceColumn = "連絡先種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@電話種別コード", DbType.AnsiString, YCDbColumnLength.TelephoneTypeCode).SourceColumn = "電話種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@電話番号", DbType.AnsiString, YCDbColumnLength.TelephoneNo).SourceColumn = "電話番号";
            YCDbHelper.AddParameter(this.Connection, parameters, "@有効時間帯メモ", DbType.AnsiString, 100).SourceColumn = "有効時間帯メモ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@利用不可フラグ", DbType.Int16).SourceColumn = "利用不可フラグ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@備考", DbType.AnsiString, YCDbColumnLength.Note).SourceColumn = "備考";

            return sql;
        }

        /// <summary>
        /// UPDATE文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>UPDATE文</returns>
        protected override string MakeSQLUpdate(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "UPDATE Ｍ顧客電話 SET";
            sql += " 連絡先種別コード = @連絡先種別コード";
            sql += ", 電話種別コード = @電話種別コード";
            sql += ", 電話番号 = @電話番号";
            sql += ", 有効時間帯メモ = @有効時間帯メモ";
            sql += ", 利用不可フラグ = @利用不可フラグ";
            sql += ", 備考 = @備考";
            sql += " WHERE 顧客電話コード = @顧客電話コード;";

            YCDbHelper.AddParameter(this.Connection, parameters, "@連絡先種別コード", DbType.AnsiString, YCDbColumnLength.ContactTypeCode).SourceColumn = "連絡先種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@電話種別コード", DbType.AnsiString, YCDbColumnLength.TelephoneTypeCode).SourceColumn = "電話種別コード";
            YCDbHelper.AddParameter(this.Connection, parameters, "@電話番号", DbType.AnsiString, YCDbColumnLength.TelephoneNo).SourceColumn = "電話番号";
            YCDbHelper.AddParameter(this.Connection, parameters, "@有効時間帯メモ", DbType.AnsiString, 100).SourceColumn = "有効時間帯メモ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@利用不可フラグ", DbType.Int16).SourceColumn = "利用不可フラグ";
            YCDbHelper.AddParameter(this.Connection, parameters, "@備考", DbType.AnsiString, YCDbColumnLength.Note).SourceColumn = "備考";
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客電話コード", DbType.Guid).SourceColumn = "顧客電話コード";

            return sql;
        }

        /// <summary>
        /// DELETE文の生成
        /// </summary>
        /// <param name="parameters">パラメータ</param>
        /// <returns>DELETE文</returns>
        protected override string MakeSQLDelete(System.Data.Common.DbParameterCollection parameters)
        {
            string sql = "DELETE FROM Ｍ顧客電話 WHERE 顧客電話コード = @顧客電話コード;";
            
            YCDbHelper.AddParameter(this.Connection, parameters, "@顧客電話コード", DbType.Guid).SourceColumn = "顧客電話コード";

            return sql;
        }

        /// <summary>
        /// データコンテナを利用した追加
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public override void Add(YCDataContainer container)
        {
            DataRow row = null;

            try
            {
                row = this.NewRow;
                row["顧客電話コード"] = YCAutoNoManager.Instance.GetGuid();
                row["連絡先種別コード"] = container["連絡先種別コード"];
                if(YCConvert.ToString(row["連絡先種別コード"]) != "")
                {
                    row["連絡先種別名"] = YCListManager.Instance.Lists["連絡先種別"][YCConvert.ToString(row["連絡先種別コード"])].Value;
                }
                row["電話種別コード"] = container["電話種別コード"];
                if (YCConvert.ToString(row["電話種別コード"]) != "")
                {
                    row["電話種別名"] = YCListManager.Instance.Lists["電話種別"][YCConvert.ToString(row["電話種別コード"])].Value;
                }
                row["電話番号"] = container["電話番号"];
                row["有効時間帯メモ"] = container["有効時間帯メモ"];
                row["利用不可フラグ"] = container["利用不可フラグ"];
                row["備考"] = container["備考"];

                this.Table.Rows.Add(row);
                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// 電話番号のみでの追加
        /// </summary>
        /// <param name="data">電話番号</param>
        public override void Add(string data)
        {
            DataRow row = null;

            try
            {
                row = this.NewRow;
                row["顧客電話コード"] = YCAutoNoManager.Instance.GetGuid();
                row["電話番号"] = data;
                row["利用不可フラグ"] = 0;

                this.Table.Rows.Add(row);
                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// データコンテナを利用した変更
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public override void Update(YCDataContainer container)
        {
            if (YCConvert.ToGuid(container["顧客電話コード"]) == Guid.Empty) throw new YCException("顧客電話コードが不正です。");

            foreach(DataRow row in this.Table.Rows)
            {
                if(YCConvert.ToGuid(row["顧客電話コード"]) == YCConvert.ToGuid(container["顧客電話コード"]))
                {
                    row["連絡先種別コード"] = container["連絡先種別コード"];
                    if (YCConvert.ToString(row["連絡先種別コード"]) != "")
                    {
                        row["連絡先種別名"] = YCListManager.Instance.Lists["連絡先種別"][YCConvert.ToString(row["連絡先種別コード"])].Value;
                    }
                    row["電話種別コード"] = container["電話種別コード"];
                    if (YCConvert.ToString(row["電話種別コード"]) != "")
                    {
                        row["電話種別名"] = YCListManager.Instance.Lists["電話種別"][YCConvert.ToString(row["電話種別コード"])].Value;
                    }
                    row["電話番号"] = container["電話番号"];
                    row["有効時間帯メモ"] = container["有効時間帯メモ"];
                    row["利用不可フラグ"] = container["利用不可フラグ"];
                    row["備考"] = container["備考"];

                    break;
                }
            }
        }

        /// <summary>
        /// 電話番号のみの変更
        /// </summary>
        /// <param name="code">顧客電話コード</param>
        /// <param name="data">電話番号</param>
        public override void Update(Guid code, string data)
        {
            if (code == Guid.Empty) throw new YCException("顧客電話コードが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["顧客電話コード"]) == code)
                {
                    row["電話番号"] = data;
                    break;
                }
            }
        }

        /// <summary>
        /// 削除
        /// </summary>
        /// <param name="code">顧客電話コード</param>
        public override void Delete(Guid code)
        {
            if (code == Guid.Empty) throw new YCException("顧客電話コードが不正です。");

            foreach (DataRow row in this.Table.Rows)
            {
                if (YCConvert.ToGuid(row["顧客電話コード"]) == code)
                {
                    row.Delete();
                    break;
                }
            }
        }

    }
}
