﻿using System;
using System.Collections.Generic;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// リストマネージャクラス
    /// Singletonパターンです。
    /// </summary>
    public class YCListManager : YCListManagerBase
    {
        private static YCListManager _instance = null;

        private YCListManager()
            : base()
        {
        }

        public override void Dispose()
        {
            if (this.IsDisposable)
            {
                _instance = null;
            }
            base.Dispose();
        }

        //Singletonのインスタンスです。
        public static YCListManager Instance
        {
            get
            {
                if (_instance == null) _instance = new YCListManager();
                return _instance;
            }
        }

        /// <summary>
        /// 入力用のリスト（コードと名称のリスト）を読み込みます。
        /// </summary>
        public override void LoadLists()
        {
            //下記はサンプル
            try
            {
                this.OpenDialog();
                int count = 0;

                //ユーザグループ
                this.DisplayTargetListInfo("ユーザグループ", ++count);
                string sql = "SELECT Ｍユーザグループ.ユーザグループコード, Ｍユーザグループ.ユーザグループ名";
                sql += ", Ｍユーザグループ.管理者フラグ, Ｍユーザグループ.ゲストフラグ";
                sql += ", Ｍユーザグループ.権限レベル";
                sql += " FROM Ｍユーザグループ";
                sql += " WHERE (Ｍユーザグループ.削除フラグ = 0 OR Ｍユーザグループ.削除フラグ IS NULL)";
                sql += " ORDER BY Ｍユーザグループ.プライオリティ;";
                int idx = this.Lists.Add(new YCCodeList("ユーザグループ"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "ユーザグループコード";
                this.Lists[idx].ValueColumnName = "ユーザグループ名";
                this.Lists[idx].SubItemColumnNameList.Add("管理者フラグ");
                this.Lists[idx].SubItemColumnNameList.Add("ゲストフラグ");
                this.Lists[idx].SubItemColumnNameList.Add("権限レベル");
                this.Lists[idx].Load();

                //法個人
                this.DisplayTargetListInfo("法個人", ++count);
                sql = "SELECT Ｍ法個人.法個人コード, Ｍ法個人.法個人名";
                sql += ", Ｍ法個人.法個人名カナ, Ｍ法個人.法人フラグ";
                sql += ", Ｍ法個人.法個人略称, Ｍ法個人.法個人略称カナ";
                sql += " FROM Ｍ法個人";
                sql += " ORDER BY Ｍ法個人.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("法個人"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "法個人コード";
                this.Lists[idx].ValueColumnName = "法個人名";
                this.Lists[idx].SubItemColumnNameList.Add("法個人名カナ");
                this.Lists[idx].SubItemColumnNameList.Add("法人フラグ");
                this.Lists[idx].SubItemColumnNameList.Add("法個人略称");
                this.Lists[idx].SubItemColumnNameList.Add("法個人略称カナ");
                this.Lists[idx].Load();

                //法人格配置
                this.DisplayTargetListInfo("法人格配置", ++count);
                sql = "SELECT Ｍ法人格配置.法人格配置コード, Ｍ法人格配置.法人格配置名";
                sql += " FROM Ｍ法人格配置";
                sql += " ORDER BY Ｍ法人格配置.法人格配置コード;";
                idx = this.Lists.Add(new YCCodeList("法人格配置"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "法人格配置コード";
                this.Lists[idx].ValueColumnName = "法人格配置名";
                this.Lists[idx].Load();

                //都道府県
                this.DisplayTargetListInfo("都道府県", ++count);
                sql = "SELECT Ｍ都道府県.都道府県コード, Ｍ都道府県.都道府県名";
                sql += ",Ｍ都道府県.都道府県名カナ";
                sql += " FROM Ｍ都道府県";
                sql += " ORDER BY Ｍ都道府県.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("都道府県"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "都道府県コード";
                this.Lists[idx].ValueColumnName = "都道府県名";
                this.Lists[idx].SubItemColumnNameList.Add("都道府県名カナ");
                this.Lists[idx].Load();

                //業種
                this.DisplayTargetListInfo("業種", ++count);
                sql = "SELECT Ｍ業種.業種コード, Ｍ業種.業種名";
                sql += " FROM Ｍ業種";
                sql += " ORDER BY Ｍ業種.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("業種"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "業種コード";
                this.Lists[idx].ValueColumnName = "業種名";
                this.Lists[idx].Load();

                //職種
                this.DisplayTargetListInfo("職種", ++count);
                sql = "SELECT Ｍ職種.職種コード, Ｍ職種.職種名";
                sql += " FROM Ｍ職種";
                sql += " ORDER BY Ｍ職種.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("職種"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "職種コード";
                this.Lists[idx].ValueColumnName = "職種名";
                this.Lists[idx].Load();

                //電話種別
                this.DisplayTargetListInfo("電話種別", ++count);
                sql = "SELECT Ｍ電話種別.電話種別コード, Ｍ電話種別.電話種別名";
                sql += " FROM Ｍ電話種別";
                sql += " ORDER BY Ｍ電話種別.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("電話種別"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "電話種別コード";
                this.Lists[idx].ValueColumnName = "電話種別名";
                this.Lists[idx].Load();

                //メール種別
                this.DisplayTargetListInfo("メール種別", ++count);
                sql = "SELECT Ｍメール種別.メール種別コード, Ｍメール種別.メール種別名";
                sql += " FROM Ｍメール種別";
                sql += " ORDER BY Ｍメール種別.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("メール種別"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "メール種別コード";
                this.Lists[idx].ValueColumnName = "メール種別名";
                this.Lists[idx].Load();

                //連絡先種別
                this.DisplayTargetListInfo("連絡先種別", ++count);
                sql = "SELECT Ｍ連絡先種別.連絡先種別コード, Ｍ連絡先種別.連絡先種別名";
                sql += ", Ｍ連絡先種別.会社フラグ";
                sql += " FROM Ｍ連絡先種別";
                sql += " ORDER BY Ｍ連絡先種別.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("連絡先種別"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "連絡先種別コード";
                this.Lists[idx].ValueColumnName = "連絡先種別名";
                this.Lists[idx].SubItemColumnNameList.Add("会社フラグ");
                this.Lists[idx].Load();

                //メールアカウント
                this.DisplayTargetListInfo("メールアカウント", ++count);
                sql = "SELECT Ｍメールアカウント.メールアカウントコード, Ｍメールアカウント.メールアカウント名";
                sql += ", Ｍメールアカウント.アカウントＩＤ, Ｍメールアカウント.パスワード";
                sql += ", Ｍメールアカウント.メールアドレス, Ｍメールアカウント.メールアドレス表示名";
                sql += ", Ｍメールアカウント.ＳＭＴＰサーバ, Ｍメールアカウント.ＳＭＴＰポート";
                sql += ", Ｍメールアカウント.ＳＭＴＰ認証フラグ";
                sql += ", Ｍメールアカウント.ＳＭＴＰ認証アカウントＩＤ, Ｍメールアカウント.ＳＭＴＰ認証パスワード";
                sql += " FROM Ｍメールアカウント";
                sql += " ORDER BY Ｍメールアカウント.プライオリティ;";
                idx = this.Lists.Add(new YCCodeList("メールアカウント"));
                this.Lists[idx].SQL = sql;
                this.Lists[idx].KeyColumnName = "メールアカウントコード";
                this.Lists[idx].ValueColumnName = "メールアカウント名";
                this.Lists[idx].SubItemColumnNameList.Add("アカウントＩＤ");
                this.Lists[idx].SubItemColumnNameList.Add("パスワード");
                this.Lists[idx].SubItemColumnNameList.Add("メールアドレス");
                this.Lists[idx].SubItemColumnNameList.Add("メールアドレス表示名");
                this.Lists[idx].SubItemColumnNameList.Add("ＳＭＴＰサーバ");
                this.Lists[idx].SubItemColumnNameList.Add("ＳＭＴＰポート");
                this.Lists[idx].SubItemColumnNameList.Add("ＳＭＴＰ認証フラグ");
                this.Lists[idx].SubItemColumnNameList.Add("ＳＭＴＰ認証アカウントＩＤ");
                this.Lists[idx].SubItemColumnNameList.Add("ＳＭＴＰ認証パスワード");
                this.Lists[idx].Load();

                this.CloseDialog();
            }
            catch (Exception e)
            {
                try { this.CloseDialog(); }
                catch { }
                throw e;
            }
        }
    }
}
