﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

using YSystem;
using YSystem.Tools;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客名検索リスト
    /// </summary>
    public class YCSearchCustomerNameList : YCSearchNameList
    {
        public YCSearchCustomerNameList(YCConnection connection)
            : base(connection)
        {
        }

        /// <summary>
        /// 検索対象テーブル（SELECT文のFROM句）
        /// </summary>
        public override string TargetTables
        {
            get
            {
                return "Ｍ顧客 LEFT JOIN Ｍ都道府県 ON Ｍ顧客.都道府県コード = Ｍ都道府県.都道府県コード";
            }
        }

        /// <summary>
        /// 名称検索項目、名称カナ検索項目のあるテーブル名
        /// <para>TargetTablesに複数テーブルが使われている場合は指定が必要</para>
        /// </summary>
        public override string NameColumnTableName
        {
            get
            {
                return "Ｍ顧客";
            }
        }

        /// <summary>
        /// 名称検索の対象となる項目名
        /// </summary>
        public override string NameColumnName
        {
            get
            {
                return "顧客名";
            }
        }

        /// <summary>
        /// 名称カナ検索の対象となる項目名
        /// </summary>
        public override string NameKanaColumnName
        {
            get
            {
                return "顧客名カナ"; 
            }
        }

        /// <summary>
        /// 主キー項目名
        /// </summary>
        public override string KeyColumnName
        {
            get
            {
                return "顧客コード";
            }
        }

        /// <summary>
        /// 主キー項目のあるテーブル名
        /// <para>TargetTablesに複数テーブルが使われている場合は指定が必要</para>
        /// </summary>
        public override string KeyColumnTableName
        {
            get
            {
                return "Ｍ顧客";
            }
        }

        /// <summary>
        /// 主キー、名称、名称カナ以外にリストに追加する項目
        /// </summary>
        protected override string AdditionalColumns
        {
            get
            {
                string ans = "(ISNULL(Ｍ都道府県.都道府県名, '') + ISNULL(市区町村, '') + ISNULL(番地, '') + ' ' + ISNULL(ビル等, '')) AS 住所";
                ans += ", Ｍ顧客.無効フラグ";
                return ans;
            }
        }

        public override void RefreshRow(YCDbObject dbObject)
        {
            DataRow row = null;

            try
            {
                if (dbObject is YCCustomer)
                {
                    int index = this.StringDataToIndex(dbObject.KeyColumnName, dbObject.KeyCode);
                    if (index != YCDatabaseConst.NO_INDEX)
                    {
                        row = this.Table.Rows[index];
                    }
                    else
                    {
                        row = this.NewRow;
                    }

                    row["顧客コード"] = dbObject["顧客コード"];
                    row["顧客名"] = dbObject["顧客名"];
                    row["顧客名カナ"] = dbObject["顧客名カナ"];
                    row["住所"] = ((YCCustomer)dbObject).Address;
                    row["無効フラグ"] = dbObject["無効フラグ"];

                    if (index == YCDatabaseConst.NO_INDEX) this.Table.Rows.Add(row);

                    row = null;
                }
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        /// <summary>
        /// 指定の顧客を削除します。
        /// </summary>
        /// <param name="customerCode">顧客コード</param>
        public void DeleteCustomer(string customerCode)
        {
            YCCustomer target = null;

            try
            {
                target = new YCCustomer(this.Connection);
                target.KeyCode = customerCode;
                target.Delete();

                int index = this.StringDataToIndex("顧客コード", customerCode);
                if (index != YCDatabaseConst.NO_INDEX)
                {
                    this.Table.Rows[index].Delete();
                }

            }
            catch (Exception e)
            {

                if (target != null)
                {
                    try { target.Dispose(); }
                    catch { }
                    target = null;
                }

                throw e;
            }
        }
    }
}
