﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// システム情報ヘルパクラス
    /// <para>ConfigやLocalSettingsの情報を取得します。</para>
    /// </summary>
    public abstract class YCSystemInformationHelper
    {
        /// <summary>
        /// 一時ファイル格納用フォルダの取得
        /// </summary>
        /// <returns>フォルダ（フルパス）</returns>
        public static string GetTemporaryFolder()
        {
            //Config.xmlからの読込み
            return YCApplication.Instance.Config.GetKey("application", "temporary_folder");
        }

        /// <summary>
        /// CSVファイル出力用フォルダの取得
        /// </summary>
        /// <returns>フォルダ（フルパス）</returns>
        public static string GetOutputCsvFolder()
        {
            //LocalSettings.xmlからの読込み
            string folder = YCApplication.Instance.LocalSettings.GetKey("application", "output_csv_folder");

            if (folder == "")
            {
                folder = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                YCSystemInformationHelper.SetOutputCsvFolder(folder);
            }

            return folder;
        }

        /// <summary>
        /// CSVファイル出力用フォルダの設定
        /// </summary>
        /// <param name="folder">フォルダ（フルパス）</param>
        public static void SetOutputCsvFolder(string folder)
        {
            //LocalSettings.xmlへの保存
            YCApplication.Instance.LocalSettings.SetKey("application", "output_csv_folder", folder);
            try { YCApplication.Instance.LocalSettings.Save(); }
            catch { }
        }
    }
}
