﻿using System;
using System.Collections.Generic;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// My顧客管理のユーザクラス
    /// </summary>
    public class YCUser : YCUserBase
    {
        public YCUser()
            : base()
        {
        }

        //ユーザ毎のメニュー画面の型を返してください。
        public override Type MainMenuType
        {
            get
            {
                if (YCApplication.Instance.User.IsAdministrator)
                {
                    //管理者メニュー
                    return typeof(frmAdministratorMenu);
                }
                else
                {
                    //一般ユーザメニュー
                    return typeof(frmOrdinaryUserMenu);
                }
            }
        }

        //データ権限チェック用のコードです。
        //データ権限チェックの機能が不要の場合は空文字を返してください。
        public override string AuthorityCode
        {
            get
            {
                return YCConvert.ToString(this["ユーザグループコード"]);    
            }
        }

        //ユーザ情報取得用のSQL文を返します。
        //ユーザ情報の取得が必要ない場合は空文字を返してください。
        protected override string SQL
        {
            get
            {
                //throw new YCException("YCUser.SQLを実装してください。");

                //下記はサンプル
                string ans = "SELECT Ｍユーザ.ユーザＩＤ, Ｍユーザ.ユーザ名";
                ans += ", Ｍユーザグループ.ユーザグループコード, Ｍユーザグループ.権限レベル";
                ans += ", Ｍユーザグループ.管理者フラグ, Ｍユーザグループ.ゲストフラグ";
                ans += " FROM Ｍユーザ WITH(NOLOCK)";
                ans += " LEFT JOIN Ｍユーザグループ WITH(NOLOCK) ON Ｍユーザ.ユーザグループコード = Ｍユーザグループ.ユーザグループコード";
                ans += " WHERE " + this.SQLUserIDCondition + ";";
                
                return ans;
            }
        }

        //ユーザが管理者権限の場合はtrue
        public override bool IsAdministrator
        {
            get
            {
                return YCConvert.IntegerToBoolean(this["管理者フラグ"]);
            }
        }

        //ユーザがゲスト権限の場合はtrue
        public override bool IsGuest
        {
            get
            {
                return YCConvert.IntegerToBoolean(this["ゲストフラグ"]);
            }
        }

        /// <summary>
        /// 顧客リストの印刷ができるかどうかを取得
        /// <para>権限レベルが一定以上のユーザはtrue</para>
        /// </summary>
        public bool CanPrintCustomerList
        {
            get
            {
                if (YCConvert.ToInt(this["権限レベル"]) >= YCUserLevel.CanPrintCustomerList)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        //スタティックメソッド
        /// <summary>
        /// 現在ログオンしているユーザを取得します。
        /// <para>YCApplication.Instance.UserをYCUserにキャストしています。</para>
        /// </summary>
        public static YCUser CurrentUser
        {
            get
            {
                return (YCUser)YCApplication.Instance.User;
            }
        }

    }
}
