﻿using System;
using System.Collections.Generic;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// ユーザ登録用クラス
    /// </summary>
    public class YCUserForRegistration : YCDbObject
    {
        private const int USER_ID_LENGTH = 20;

        public YCUserForRegistration(YCConnection connection)
            : base(connection)
        {
        }

        //キー項目名
        public override string KeyColumnName
        {
            get
            {
                return "ユーザＩＤ";
            }
        }

        //キー項目の桁数
        protected override int KeyColumnLength
        {
            get
            {
                return USER_ID_LENGTH;
            }
        }

        //テーブル名
        public override string TableName
        {
            get
            {
                return "Ｍユーザ";
            }
        }

        //キー項目（ユーザＩＤ）のデータは自動採番しない。
        public override object GetNewKeyCode()
        {
            throw new YCDbException("ユーザＩＤが不正です。");
        }

        //新規行の初期化です。
        protected override void InitializeNewRow(System.Data.DataRow newRow)
        {
            newRow["削除フラグ"] = 0;
        }

        //エントリチェック
        public override bool EntryCheck(out string errorString)
        {
            //新規ユーザＩＤの重複チェック
            try
            {
                errorString = string.Empty;
                
                if (this.IsNew)
                {
                    if (YCConvert.ToString(this["ユーザＩＤ"]) == "")
                    {
                        errorString = "ユーザＩＤが設定されていません。";
                        return false;
                    }

                    int count = this.Connection.DCount("ユーザＩＤ", "Ｍユーザ", "ユーザＩＤ = '" + YCConvert.ToString(this["ユーザＩＤ"]) + "'");

                    if (count > 0)
                    {
                        errorString = "ユーザＩＤが重複しています。";
                        return false;
                    }
                    else return true;
                }
                else return true;
            }
            catch (Exception e)
            {
                errorString = "ユーザＩＤの確認に失敗しました。\n" + e.Message;
                return false;
            }
        }

        //ユーザグループ名の取得
        public string UserGroupName
        {
            get
            {
                string code = YCConvert.ToString(this["ユーザグループコード"]);
                if (code != "")
                {
                    return YCConvert.ToString(YCListManager.Instance.Lists["ユーザグループ"][code].Value);
                }
                else
                {
                    return string.Empty;
                }
            }
        }
    }
}
