﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// ユーザリスト
    /// </summary>
    public class YCUserList : YCDbList
    {
        public YCUserList(YCConnection connection)
            : base(connection)
        {
        }

        //SELECT文
        protected override string MakeSQLSelect(DbParameterCollection parameters)
        {
            string ans = "SELECT Ｍユーザ.ユーザＩＤ, Ｍユーザ.ユーザ名, Ｍユーザ.ユーザ名カナ";
            ans += ", Ｍユーザ.パスワード, Ｍユーザグループ.ユーザグループ名, Ｍユーザ.削除フラグ";
            ans += " FROM Ｍユーザ WITH(NOLOCK)";
            ans += " LEFT JOIN Ｍユーザグループ WITH(NOLOCK) ON Ｍユーザ.ユーザグループコード = Ｍユーザグループ.ユーザグループコード";
            ans += " ORDER BY Ｍユーザ.ユーザＩＤ;";
            
            return ans;
        }

        //追加・変更データをリストに反映します。
        public override void RefreshRow(YCDbObject dbObject)
        {
            DataRow row = null;

            try
            {
                int index = this.StringDataToIndex(dbObject.KeyColumnName, dbObject.KeyCode);
                if (index != YCDatabaseConst.NO_INDEX)
                {
                    row = this.Table.Rows[index];
                }
                else
                {
                    row = this.NewRow;
                }

                row["ユーザＩＤ"] = dbObject["ユーザＩＤ"];
                row["ユーザ名"] = dbObject["ユーザ名"];
                row["ユーザ名カナ"] = dbObject["ユーザ名カナ"];
                row["パスワード"] = dbObject["パスワード"];
                row["ユーザグループ名"] = ((YCUserForRegistration)dbObject).UserGroupName;
                row["削除フラグ"] = dbObject["削除フラグ"];

                if (index == YCDatabaseConst.NO_INDEX) this.Table.Rows.Add(row);

                row = null;
            }
            catch (Exception e)
            {
                row = null;
                throw e;
            }
        }

        //ファイル出力サンプル
        public override void WriteToFile(YCFile file)
        {
            try
            {
                if (this.RowCount < 1) return;

                file.Open();

                this.MoveFirst();
                while (!this.EOF)
                {
                    string data = "\"" + YCConvert.ToString(this["ユーザＩＤ"]) + "\"";
                    data += ",\"" + YCConvert.ToString(this["ユーザ名"]) + "\"";
                    data += ",\"" + YCConvert.ToString(this["ユーザ名カナ"]) + "\"";
                    data += ",\"" + YCConvert.ToString(this["パスワード"]) + "\"";
                    data += ",\"" + YCConvert.ToString(this["ユーザグループ名"]) + "\"";
                    data += "," + YCConvert.ToString(this["削除フラグ"]);

                    file.WriteLine(data);

                    this.MoveNext();
                }

                file.Close();
            }
            catch (Exception e)
            {
                if (file.IsOpened) file.Close();
                throw e;
            }
        }
    }
}
