﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 業種登録用リストダイアログ
    /// </summary>
    public partial class dlgBusinessTypeList : YCDbListDialog
    {
        public dlgBusinessTypeList()
        {
            InitializeComponent();
        }

        /// <summary>
        /// データグリッドビュー
        /// </summary>
        protected override DataGridView ListGrid
        {
            get
            {
                return this.dgList;
            }
        }

        /// <summary>
        /// データグリッドビューの列の作成
        /// </summary>
        protected override void CreateListGridColumns()
        {
            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //項目にIMEモードの指定ができます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddGridTextColumn("業種コード", ImeMode.Off);
                this.AddGridTextColumn("業種名", ImeMode.Hiragana);
                this.AddGridTextColumn("プライオリティ", ImeMode.Off);
            }
            else
            {
                this.AddGridTextColumn("業種コード");
                this.AddGridTextColumn("業種名");
                this.AddGridTextColumn("プライオリティ");
            }

        }

        /// <summary>
        /// データグリッドビューの列の書式設定
        /// </summary>
        protected override void FormatListGrid()
        {
            string columnName;

            //dgList.ReadOnly = true;
            this.ListGrid.AllowUserToAddRows = true;
            this.ListGrid.AllowUserToDeleteRows = true;
            this.ListGrid.AllowUserToOrderColumns = false;
            this.ListGrid.AllowUserToResizeColumns = true;

            this.ListGrid.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.ListGrid.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.ListGrid.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.ListGrid.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "業種コード";
            this.ListGrid.Columns[columnName].Width = 70;
            this.ListGrid.Columns[columnName].HeaderText = "コード";

            columnName = "業種名";
            this.ListGrid.Columns[columnName].Width = 100;
            this.ListGrid.Columns[columnName].HeaderText = "業種名";

            columnName = "プライオリティ";
            this.ListGrid.Columns[columnName].Width = 70;
            this.ListGrid.Columns[columnName].HeaderText = "順位";
            this.ListGrid.Columns[columnName].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
        }

        //イベント処理
        private void dlgBusinessTypeList_Load(object sender, EventArgs e)
        {
            try
            {
                this.LoadData();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {
            try
            {
                this.EndEdit();

                YCCursor.Wait();
                this.Save(true);
                YCCursor.Default();

                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            catch (Exception err)
            {
                YCCursor.Default();
                YCMessageBox.ShowErrorMessage(err.Message);

                this.DialogResult = DialogResult.Cancel;
                this.Close();
            }
        }

    }
}
