﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;
using YSystem.Controls;
using YSystem.Tools;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客ダイアログ
    /// </summary>
    public partial class dlgCustomer : YCDbObjectDialog
    {
        private bool _subListIsLoaded = false;

        public dlgCustomer()
        {
            InitializeComponent();
        }

        //選択リストの初期化
        protected override void SetControlSelections()
        {
            YCControlHelper.FillComboBox(ctl法個人コード, YCListManager.Instance.Lists["法個人"], true);
            YCControlHelper.FillComboBox(ctl法人格配置コード, YCListManager.Instance.Lists["法人格配置"], true);
            YCControlHelper.FillComboBox(ctl業種コード, YCListManager.Instance.Lists["業種"], true);
            YCControlHelper.FillComboBox(ctl職種コード, YCListManager.Instance.Lists["職種"], true);
            YCControlHelper.FillComboBox(ctl都道府県コード, YCListManager.Instance.Lists["都道府県"], true);
        }

        //データ項目のバインド
        protected override void SetControlBindings()
        {
            this.BindControl(ctl顧客名, "Text", "顧客名");
            this.BindControl(ctl顧客名カナ, "Text", "顧客名カナ");
            this.BindControl(ctl法個人コード, "SelectedValue", "法個人コード");
            this.BindControl(ctl法人格配置コード, "SelectedValue", "法人格配置コード");
            this.BindControl(ctl業種コード, "SelectedValue", "業種コード");
            this.BindControl(ctl職種コード, "SelectedValue", "職種コード");
            this.BindControl(ctl法人担当者名, "Text", "法人担当者名");
            this.BindControl(ctl法人担当者所属, "Text", "法人担当者所属");
            this.BindControl(ctl法人担当者役職, "Text", "法人担当者役職");
            this.BindControl(ctl郵便番号, "Text", "郵便番号");
            this.BindControl(ctl都道府県コード, "SelectedValue", "都道府県コード");
            this.BindControl(ctl市区町村, "Text", "市区町村");
            this.BindControl(ctl番地, "Text", "番地");
            this.BindControl(ctlビル等, "Text", "ビル等");
            this.BindControl(ctl取引メモ, "Text", "取引メモ");
            this.BindControl(ctl備考, "Text", "備考");
        }

        //バインドしないコントロールの初期化
        //（bool型項目でないデータ項目用のチェックボックス等）
        protected override void SetUnbindControls()
        {
            this.SetCheckBoxByIntegerValue(ctl無効フラグ, "無効フラグ");
        }

        //エントリチェック
        private bool EntryCheck()
        {
            string msg;
            if (!this.Object.EntryCheck(out msg))
            {
                YCMessageBox.ShowErrorMessage(msg);
                return false;
            }
            else return true;

        }

        /// <summary>
        /// 顧客オブジェクトに従属するリストの読込み
        /// </summary>
        private void LoadSubList()
        {
            if (!_subListIsLoaded)
            {
                this.LoadTelephoneList();
                this.LoadFaxList();
                this.LoadMailList();
                this.LoadGroupList();

                _subListIsLoaded = true;
            }
        }

        #region 電話番号リスト関連
        
        /// <summary>
        /// 電話リストの読込み
        /// </summary>
        private void LoadTelephoneList()
        {
            this.CreateTelephoneListGridColumns();
            YCDataGridViewHelper.BindList(dg電話リスト, ((YCCustomer)this.Object).TelephoneList);
            this.FormatTelephoneListGrid();
        }

        /// <summary>
        /// 電話リストの項目作成
        /// </summary>
        private void CreateTelephoneListGridColumns()
        {
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "顧客電話コード");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "電話番号");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "連絡先種別コード");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "連絡先種別名");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "電話種別コード");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "電話種別名");
            YCDataGridViewHelper.AddCheckBoxColumn(dg電話リスト, "利用不可フラグ", -1, 0);
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "有効時間帯メモ");
            YCDataGridViewHelper.AddTextBoxColumn(dg電話リスト, "備考");

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //右端にダミー列を入れて最後列の幅を変更可能にできます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //
             
                YCDataGridViewHelper.AddDummyColumn(dg電話リスト);
            }
        }

        /// <summary>
        /// 電話リストの表示の初期化
        /// </summary>
        private void FormatTelephoneListGrid()
        {
            string columnName;

            this.dg電話リスト.ReadOnly = true;
            this.dg電話リスト.AllowUserToAddRows = false;
            this.dg電話リスト.AllowUserToDeleteRows = false;
            this.dg電話リスト.AllowUserToOrderColumns = false;
            this.dg電話リスト.AllowUserToResizeColumns = true;

            this.dg電話リスト.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.dg電話リスト.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.dg電話リスト.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.dg電話リスト.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "顧客電話コード";
            this.dg電話リスト.Columns[columnName].Visible = false;

            columnName = "電話番号";
            this.dg電話リスト.Columns[columnName].Width = 100;

            columnName = "連絡先種別コード";
            this.dg電話リスト.Columns[columnName].Visible = false;

            columnName = "連絡先種別名";
            this.dg電話リスト.Columns[columnName].Width = 100;
            this.dg電話リスト.Columns[columnName].HeaderText = "連絡先";

            columnName = "電話種別コード";
            this.dg電話リスト.Columns[columnName].Visible = false;

            columnName = "電話種別名";
            this.dg電話リスト.Columns[columnName].Width = 70;
            this.dg電話リスト.Columns[columnName].HeaderText = "種別";

            columnName = "利用不可フラグ";
            this.dg電話リスト.Columns[columnName].Width = 50;
            this.dg電話リスト.Columns[columnName].HeaderText = "不可";

            columnName = "有効時間帯メモ";
            this.dg電話リスト.Columns[columnName].Width = 150;
            this.dg電話リスト.Columns[columnName].HeaderText = "時間帯";

            columnName = "備考";
            this.dg電話リスト.Columns[columnName].Width = 300;

        }

        /// <summary>
        /// 顧客電話の追加
        /// </summary>
        private void AddCustomerTelephone()
        {
            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddCustomerTelephoneWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                bool ret = false;
                string telNo = YCInputTools.GetString(out ret, "電話番号入力", string.Empty, ImeMode.Off);

                if (ret)
                {
                    ((YCCustomer)this.Object).TelephoneList.Add(telNo);
                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客電話の追加
        /// </summary>
        private void AddCustomerTelephoneWithContainer()
        {
            YCCustomerTelephoneContainer container = null;

            try
            {
                container = new YCCustomerTelephoneContainer();
                container["利用不可フラグ"] = 0;
                container.IncrementRef();

                if (this.InputCustomerTelephoneInformation(container))
                {
                    ((YCCustomer)this.Object).TelephoneList.Add(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客電話の変更
        /// </summary>
        private void UpdateCustomerTelephone()
        {
            if (dg電話リスト.SelectedRows.Count == 0) return;

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.UpdateCustomerTelephoneWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "顧客電話コード"));
                if (code == Guid.Empty) return;
                string oldTelNo = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "電話番号"));

                bool ret = false;
                string telNo = YCInputTools.GetString(out ret, "電話番号入力", oldTelNo, ImeMode.Off);

                if (ret)
                {

                    ((YCCustomer)this.Object).TelephoneList.Update(code, telNo);

                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客電話の変更
        /// </summary>
        private void UpdateCustomerTelephoneWithContainer()
        {
            YCCustomerTelephoneContainer container = null;

            try
            {
                container = new YCCustomerTelephoneContainer();
                container["顧客電話コード"] = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "顧客電話コード"));
                container["連絡先種別コード"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "連絡先種別コード"));
                container["電話種別コード"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "電話種別コード"));
                container["電話番号"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "電話番号"));
                container["有効時間帯メモ"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "有効時間帯メモ"));
                container["利用不可フラグ"] = YCConvert.ToInt(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "利用不可フラグ"));
                container["備考"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "備考"));

                container.IncrementRef();

                if (this.InputCustomerTelephoneInformation(container))
                {
                    ((YCCustomer)this.Object).TelephoneList.Update(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客電話情報の入力
        /// </summary>
        /// <param name="container">データコンテナ</param>
        private bool InputCustomerTelephoneInformation(YCCustomerTelephoneContainer container)
        {
            dlgCustomerTelephoneContainer dlg = null;
            bool ans = false;

            try
            {
                dlg = new dlgCustomerTelephoneContainer();
                dlg.DataContainer = container;

                if (dlg.ShowDialog(YCApplication.Instance.MdiParent) == DialogResult.OK)
                {
                    ans = true;
                }

                dlg.Dispose();
                dlg = null;

                return ans;

            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客電話の削除
        /// </summary>
        private void DeleteCustomerTelephone()
        {
            if (dg電話リスト.SelectedRows.Count == 0) return;

            Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dg電話リスト, "顧客電話コード"));
            if (code == Guid.Empty) return;

            ((YCCustomer)this.Object).TelephoneList.Delete(code);
        }

        #endregion

        #region ＦＡＸ番号リスト関連

        /// <summary>
        /// ＦＡＸリストの読込み
        /// </summary>
        private void LoadFaxList()
        {
            this.CreateFaxListGridColumns();
            YCDataGridViewHelper.BindList(dgＦＡＸリスト, ((YCCustomer)this.Object).FaxList);
            this.FormatFaxListGrid();
        }

        /// <summary>
        /// ＦＡＸリストの項目作成
        /// </summary>
        private void CreateFaxListGridColumns()
        {
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "顧客ＦＡＸコード");
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "ＦＡＸ番号");
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "連絡先種別コード");
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "連絡先種別名");
            YCDataGridViewHelper.AddCheckBoxColumn(dgＦＡＸリスト, "利用不可フラグ", -1, 0);
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "有効時間帯メモ");
            YCDataGridViewHelper.AddTextBoxColumn(dgＦＡＸリスト, "備考");

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //右端にダミー列を入れて最後列の幅を変更可能にできます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                YCDataGridViewHelper.AddDummyColumn(dgＦＡＸリスト);
            }

        }

        /// <summary>
        /// ＦＡＸリストの表示の初期化
        /// </summary>
        private void FormatFaxListGrid()
        {
            string columnName;

            this.dgＦＡＸリスト.ReadOnly = true;
            this.dgＦＡＸリスト.AllowUserToAddRows = false;
            this.dgＦＡＸリスト.AllowUserToDeleteRows = false;
            this.dgＦＡＸリスト.AllowUserToOrderColumns = false;
            this.dgＦＡＸリスト.AllowUserToResizeColumns = true;

            this.dgＦＡＸリスト.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.dgＦＡＸリスト.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.dgＦＡＸリスト.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.dgＦＡＸリスト.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "顧客ＦＡＸコード";
            this.dgＦＡＸリスト.Columns[columnName].Visible = false;

            columnName = "ＦＡＸ番号";
            this.dgＦＡＸリスト.Columns[columnName].Width = 100;

            columnName = "連絡先種別コード";
            this.dgＦＡＸリスト.Columns[columnName].Visible = false;

            columnName = "連絡先種別名";
            this.dgＦＡＸリスト.Columns[columnName].Width = 100;
            this.dgＦＡＸリスト.Columns[columnName].HeaderText = "連絡先";

            columnName = "利用不可フラグ";
            this.dgＦＡＸリスト.Columns[columnName].Width = 50;
            this.dgＦＡＸリスト.Columns[columnName].HeaderText = "不可";

            columnName = "有効時間帯メモ";
            this.dgＦＡＸリスト.Columns[columnName].Width = 150;
            this.dgＦＡＸリスト.Columns[columnName].HeaderText = "時間帯";

            columnName = "備考";
            this.dgＦＡＸリスト.Columns[columnName].Width = 300;

        }

        /// <summary>
        /// 顧客ＦＡＸの追加
        /// </summary>
        private void AddCustomerFax()
        {
            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddCustomerFaxWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                bool ret = false;
                string faxNo = YCInputTools.GetString(out ret, "ＦＡＸ番号入力", string.Empty, ImeMode.Off);

                if (ret)
                {
                    ((YCCustomer)this.Object).FaxList.Add(faxNo);
                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客ＦＡＸの追加
        /// </summary>
        private void AddCustomerFaxWithContainer()
        {
            YCCustomerFaxContainer container = null;

            try
            {
                container = new YCCustomerFaxContainer();
                container["利用不可フラグ"] = 0;
                container.IncrementRef();

                if (this.InputCustomerFaxInformation(container))
                {
                    ((YCCustomer)this.Object).FaxList.Add(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客ＦＡＸの変更
        /// </summary>
        private void UpdateCustomerFax()
        {
            if (dgＦＡＸリスト.SelectedRows.Count == 0) return;

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.UpdateCustomerFaxWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "顧客ＦＡＸコード"));
                if (code == Guid.Empty) return;
                string oldFaxNo = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "ＦＡＸ番号"));

                bool ret = false;
                string faxNo = YCInputTools.GetString(out ret, "ＦＡＸ番号入力", oldFaxNo, ImeMode.Off);

                if (ret)
                {

                    ((YCCustomer)this.Object).FaxList.Update(code, faxNo);

                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客ＦＡＸの変更
        /// </summary>
        private void UpdateCustomerFaxWithContainer()
        {
            YCCustomerFaxContainer container = null;

            try
            {
                container = new YCCustomerFaxContainer();
                container["顧客ＦＡＸコード"] = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "顧客ＦＡＸコード"));
                container["連絡先種別コード"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "連絡先種別コード"));
                container["ＦＡＸ番号"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "ＦＡＸ番号"));
                container["有効時間帯メモ"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "有効時間帯メモ"));
                container["利用不可フラグ"] = YCConvert.ToInt(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "利用不可フラグ"));
                container["備考"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "備考"));

                container.IncrementRef();

                if (this.InputCustomerFaxInformation(container))
                {
                    ((YCCustomer)this.Object).FaxList.Update(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客ＦＡＸ情報の入力
        /// </summary>
        /// <param name="container">データコンテナ</param>
        private bool InputCustomerFaxInformation(YCCustomerFaxContainer container)
        {
            dlgCustomerFaxContainer dlg = null;
            bool ans = false;

            try
            {
                dlg = new dlgCustomerFaxContainer();
                dlg.DataContainer = container;

                if (dlg.ShowDialog(YCApplication.Instance.MdiParent) == DialogResult.OK)
                {
                    ans = true;
                }

                dlg.Dispose();
                dlg = null;

                return ans;

            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客ＦＡＸの削除
        /// </summary>
        private void DeleteCustomerFax()
        {
            if (dgＦＡＸリスト.SelectedRows.Count == 0) return;


            Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgＦＡＸリスト, "顧客ＦＡＸコード"));
            if (code == Guid.Empty) return;

            ((YCCustomer)this.Object).FaxList.Delete(code);
        }

        #endregion

        #region メールリスト関連

        /// <summary>
        /// メールリストの読込み
        /// </summary>
        private void LoadMailList()
        {
            this.CreateMailListGridColumns();
            YCDataGridViewHelper.BindList(dgメールリスト, ((YCCustomer)this.Object).MailList);
            this.FormatMailListGrid();
        }

        /// <summary>
        /// メールリストの項目作成
        /// </summary>
        private void CreateMailListGridColumns()
        {
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "顧客メールコード");
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "メールアドレス");
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "連絡先種別コード");
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "連絡先種別名");
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "メール種別コード");
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "メール種別名");
            YCDataGridViewHelper.AddCheckBoxColumn(dgメールリスト, "グループ一括メールフラグ", -1, 0);
            YCDataGridViewHelper.AddTextBoxColumn(dgメールリスト, "備考");

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //右端にダミー列を入れて最後列の幅を変更可能にできます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                YCDataGridViewHelper.AddDummyColumn(dgメールリスト);
            }

        }

        /// <summary>
        /// メールリストの表示の初期化
        /// </summary>
        private void FormatMailListGrid()
        {
            string columnName;

            this.dgメールリスト.ReadOnly = true;
            this.dgメールリスト.AllowUserToAddRows = false;
            this.dgメールリスト.AllowUserToDeleteRows = false;
            this.dgメールリスト.AllowUserToOrderColumns = false;
            this.dgメールリスト.AllowUserToResizeColumns = true;

            this.dgメールリスト.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.dgメールリスト.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.dgメールリスト.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.dgメールリスト.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "顧客メールコード";
            this.dgメールリスト.Columns[columnName].Visible = false;

            columnName = "メールアドレス";
            this.dgメールリスト.Columns[columnName].Width = 200;

            columnName = "連絡先種別コード";
            this.dgメールリスト.Columns[columnName].Visible = false;

            columnName = "連絡先種別名";
            this.dgメールリスト.Columns[columnName].Width = 100;
            this.dgメールリスト.Columns[columnName].HeaderText = "連絡先";

            columnName = "メール種別コード";
            this.dgメールリスト.Columns[columnName].Visible = false;

            columnName = "メール種別名";
            this.dgメールリスト.Columns[columnName].Width = 70;
            this.dgメールリスト.Columns[columnName].HeaderText = "種別";

            columnName = "グループ一括メールフラグ";
            this.dgメールリスト.Columns[columnName].Width = 50;
            this.dgメールリスト.Columns[columnName].HeaderText = "一括";

            columnName = "備考";
            this.dgメールリスト.Columns[columnName].Width = 300;

        }

        /// <summary>
        /// 顧客メールの追加
        /// </summary>
        private void AddCustomerMail()
        {
            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddCustomerMailWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                bool ret = false;
                string mail = YCInputTools.GetString(out ret, "メールアドレス入力", string.Empty, ImeMode.Off);

                if (ret)
                {
                    ((YCCustomer)this.Object).MailList.Add(mail);
                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客メールの追加
        /// </summary>
        private void AddCustomerMailWithContainer()
        {
            YCCustomerMailContainer container = null;

            try
            {
                container = new YCCustomerMailContainer();
                container["グループ一括メールフラグ"] = -1;
                container.IncrementRef();

                if (this.InputCustomerMailInformation(container))
                {
                    ((YCCustomer)this.Object).MailList.Add(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客メールの変更
        /// </summary>
        private void UpdateCustomerMail()
        {
            if (dgメールリスト.SelectedRows.Count == 0) return;

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.UpdateCustomerMailWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "顧客メールコード"));
                if (code == Guid.Empty) return;
                string oldMail = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "メールアドレス"));

                bool ret = false;
                string mail = YCInputTools.GetString(out ret, "メールアドレス入力", oldMail, ImeMode.Off);

                if (ret)
                {

                    ((YCCustomer)this.Object).MailList.Update(code, mail);

                }
            }
        }

        /// <summary>
        /// コンテナを利用した顧客メールの変更
        /// </summary>
        private void UpdateCustomerMailWithContainer()
        {
            YCCustomerMailContainer container = null;

            try
            {
                container = new YCCustomerMailContainer();
                container["顧客メールコード"] = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "顧客メールコード"));
                container["連絡先種別コード"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "連絡先種別コード"));
                container["メール種別コード"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "メール種別コード"));
                container["メールアドレス"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "メールアドレス"));
                container["グループ一括メールフラグ"] = YCConvert.ToInt(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "グループ一括メールフラグ"));
                container["備考"] = YCConvert.ToString(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "備考"));

                container.IncrementRef();

                if (this.InputCustomerMailInformation(container))
                {
                    ((YCCustomer)this.Object).MailList.Update(container);
                }

                container.DecrementRef();
                container = null;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客メール情報の入力
        /// </summary>
        /// <param name="container">データコンテナ</param>
        private bool InputCustomerMailInformation(YCCustomerMailContainer container)
        {
            dlgCustomerMailContainer dlg = null;
            bool ans = false;

            try
            {
                dlg = new dlgCustomerMailContainer();
                dlg.DataContainer = container;

                if (dlg.ShowDialog(YCApplication.Instance.MdiParent) == DialogResult.OK)
                {
                    ans = true;
                }

                dlg.Dispose();
                dlg = null;

                return ans;

            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客メールの削除
        /// </summary>
        private void DeleteCustomerMail()
        {
            if (dgメールリスト.SelectedRows.Count == 0) return;


            Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgメールリスト, "顧客メールコード"));
            if (code == Guid.Empty) return;

            ((YCCustomer)this.Object).MailList.Delete(code);
        }

        #endregion

        #region グループリスト関連

        /// <summary>
        /// グループリストの読込み
        /// </summary>
        private void LoadGroupList()
        {
            this.CreateGroupListGridColumns();
            YCDataGridViewHelper.BindList(dgグループリスト, ((YCCustomer)this.Object).GroupList);
            this.FormatGroupListGrid();
        }

        /// <summary>
        /// グループリストの項目作成
        /// </summary>
        private void CreateGroupListGridColumns()
        {
            YCDataGridViewHelper.AddTextBoxColumn(dgグループリスト, "ＳＥＱＮｏ");
            YCDataGridViewHelper.AddTextBoxColumn(dgグループリスト, "顧客グループコード");
            YCDataGridViewHelper.AddTextBoxColumn(dgグループリスト, "顧客グループ名");

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //右端にダミー列を入れて最後列の幅を変更可能にできます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                YCDataGridViewHelper.AddDummyColumn(dgＦＡＸリスト);
            }

        }

        /// <summary>
        /// グループリストの表示の初期化
        /// </summary>
        private void FormatGroupListGrid()
        {
            string columnName;

            this.dgグループリスト.ReadOnly = true;
            this.dgグループリスト.AllowUserToAddRows = false;
            this.dgグループリスト.AllowUserToDeleteRows = false;
            this.dgグループリスト.AllowUserToOrderColumns = false;
            this.dgグループリスト.AllowUserToResizeColumns = true;

            this.dgグループリスト.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.dgグループリスト.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.dgグループリスト.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.dgグループリスト.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "ＳＥＱＮｏ";
            this.dgグループリスト.Columns[columnName].Visible = false;

            columnName = "顧客グループコード";
            this.dgグループリスト.Columns[columnName].Visible = false;

            columnName = "顧客グループ名";
            this.dgグループリスト.Columns[columnName].Width = 200;


        }

        /// <summary>
        /// 顧客グループの追加
        /// </summary>
        private void AddCustomerGroup()
        {
            string code = string.Empty;
            string name = string.Empty;

            if (YCApplicationInputTools.SelectCustomerGroupCode(out code, out name))
            {
                ((YCCustomer)this.Object).GroupList.Add(code, name);
            }
        }

        /// <summary>
        /// 顧客グループの削除
        /// </summary>
        private void DeleteCustomerGroup()
        {
            if (dgグループリスト.SelectedRows.Count == 0) return;


            Guid code = YCConvert.ToGuid(YCDataGridViewHelper.GetSelectedValue(dgグループリスト, "ＳＥＱＮｏ"));
            if (code == Guid.Empty) return;

            ((YCCustomer)this.Object).GroupList.Delete(code);
        }

        #endregion

        //イベント処理
        private void dlgCustomer_Load(object sender, EventArgs e)
        {
            try
            {
                this.LoadData();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {
            try
            {
                this.EndEdit();

                YCCursor.Wait();

                if (this.EntryCheck())
                {
                    this.Save();
                    YCCursor.Default();

                    this.DialogResult = DialogResult.OK;
                    this.Close();
                }
                else
                {
                    YCCursor.Default();
                }
            }
            catch (Exception err)
            {
                YCCursor.Default();
                YCMessageBox.ShowErrorMessage(err.Message);

                this.DialogResult = DialogResult.Cancel;
                this.Close();
            }
        }

        #region 情報Iイベント
        private void ctl無効フラグ_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                this.WriteCheckBoxByIntegerValue(ctl無効フラグ, "無効フラグ");
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void ctl顧客名_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                //顧客名カナに自動入力する。
                YCIme.KeyDownEdit(ctl顧客名, ctl顧客名カナ, ImeMode.KatakanaHalf);
            }
            catch { }
        }

        #endregion

        #region 情報IIイベント

        private void tp情報II_Enter(object sender, EventArgs e)
        {
            try
            {
                this.EndEdit();

                bool saveFlag = false;

                if (this.Object.IsNew)
                {
                    if (YCMessageBox.ShowOKCancelMessage("保存します。", true) == DialogResult.OK)
                    {
                        saveFlag = true;
                    }
                    else
                    {
                        tab顧客.SelectedIndex = 0;
                        return;
                    }
                }

                YCCursor.Wait();
                if (saveFlag) this.Save();
                this.LoadSubList();
                YCCursor.Default();
            }
            catch (Exception err)
            {
                YCCursor.Default();
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdAddTel_Click(object sender, EventArgs e)
        {
            try
            {
                this.AddCustomerTelephone();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdChangeTel_Click(object sender, EventArgs e)
        {
            try
            {
                this.UpdateCustomerTelephone();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDeleteTel_Click(object sender, EventArgs e)
        {
            try
            {
                if (YCMessageBox.ShowOKCancelMessage("電話情報を削除します。", true) == DialogResult.OK)
                {
                    this.DeleteCustomerTelephone();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdAddFax_Click(object sender, EventArgs e)
        {
            try
            {
                this.AddCustomerFax();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdChangeFax_Click(object sender, EventArgs e)
        {
            try
            {
                this.UpdateCustomerFax();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDeleteFax_Click(object sender, EventArgs e)
        {
            try
            {
                if (YCMessageBox.ShowOKCancelMessage("ＦＡＸ情報を削除します。", true) == DialogResult.OK)
                {
                    this.DeleteCustomerFax();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdAddMail_Click(object sender, EventArgs e)
        {
            try
            {
                this.AddCustomerMail();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdChangMail_Click(object sender, EventArgs e)
        {
            try
            {
                this.UpdateCustomerMail();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDeleteMail_Click(object sender, EventArgs e)
        {
            try
            {
                if (YCMessageBox.ShowOKCancelMessage("メール情報を削除します。", true) == DialogResult.OK)
                {
                    this.DeleteCustomerMail();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdAddGroup_Click(object sender, EventArgs e)
        {
            try
            {
                this.AddCustomerGroup();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDeleteGroup_Click(object sender, EventArgs e)
        {
            try
            {
                if (YCMessageBox.ShowOKCancelMessage("グループを削除します。", true) == DialogResult.OK)
                {
                    this.DeleteCustomerGroup();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        #endregion

    }
}
