﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;
using YSystem.Tools;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客グループ登録用リストフォーム
    /// </summary>
    public partial class frmCustomerGroupEntryList : YCDbListForm
    {
        private const string TITLE_BASE = "顧客グループリスト";

        //顧客グループ変更後に呼び出すイベント
        public event YMyCustomer.CustomerGroupHandler UpdatedCustomerGroup;
        
        //顧客グループ削除後に呼び出すイベント
        public event YMyCustomer.CustomerGroupHandler DeletedCustomerGroup;

        //フォームのDispose時に呼び出すイベント
        public event YMyCustomer.DisposedCustomerGroupEntryFormHandler FormDisposed;

        public frmCustomerGroupEntryList()
        {
            InitializeComponent();
        }

        /// <summary>
        /// リソースを解放します。
        /// </summary>
        private void DisposeForm()
        {
            if (!this.IsDisposed)
            {
                //フォームのDisposeメッセージを呼び出します。
                try { if (this.FormDisposed != null) this.FormDisposed(this); }
                catch { }

                //各イベントのクリア
                this.ClearEvents();
            }
        }

        /// <summary>
        /// データグリッドビュー
        /// </summary>
        protected override DataGridView ListGrid
        {
            get
            {
                return this.dgList;
            }
        }

        /// <summary>
        /// データグリッドビューの列の作成
        /// </summary>
        protected override void CreateListGridColumns()
        {

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework キーが設定されている場合
                //項目にIMEモードの指定ができます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddGridTextColumn("顧客グループコード");
                this.AddGridTextColumn("所属顧客グループコード");
                this.AddGridTextColumn("顧客グループ名", ImeMode.Hiragana);
                this.AddGridTextColumn("プライオリティ", ImeMode.Off);
            }
            else
            {
                this.AddGridTextColumn("顧客グループコード");
                this.AddGridTextColumn("所属顧客グループコード");
                this.AddGridTextColumn("顧客グループ名");
                this.AddGridTextColumn("プライオリティ");
            }
        }

        /// <summary>
        /// データグリッドビューの列の書式設定
        /// </summary>
        protected override void FormatListGrid()
        {
            string columnName;

            dgList.ReadOnly = true;
            this.ListGrid.AllowUserToAddRows = false;
            this.ListGrid.AllowUserToDeleteRows = false;
            this.ListGrid.AllowUserToOrderColumns = false;
            this.ListGrid.AllowUserToResizeColumns = true;

            this.ListGrid.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.ListGrid.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.ListGrid.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.ListGrid.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "顧客グループコード";
            this.ListGrid.Columns[columnName].Visible = false;

            columnName = "所属顧客グループコード";
            this.ListGrid.Columns[columnName].Visible = false;

            columnName = "顧客グループ名";
            this.ListGrid.Columns[columnName].Width = 300;
            this.ListGrid.Columns[columnName].HeaderText = "グループ名";

            columnName = "プライオリティ";
            this.ListGrid.Columns[columnName].Width = 70;
            this.ListGrid.Columns[columnName].HeaderText = "順位";
            this.ListGrid.Columns[columnName].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
        }

        /// <summary>
        /// フォームタイトルの設定
        /// </summary>
        private void SetTitle()
        {
            this.Text = TITLE_BASE;
            if (!((YCCustomerGroupEntryList)this.List).ParentIsRoot)
            {
                this.Text += "（" + ((YCCustomerGroupEntryList)this.List).ParentName + "）";
            }
        }

        /// <summary>
        /// 顧客グループリストを開きます。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        private void OpenCustomerGroupEntryList(string code)
        {
            YCCustomerGroupEntryList list = null;
            frmCustomerGroupEntryList frm = null;

            if (code == "") return;

            try
            {
                list = new YCCustomerGroupEntryList(YCConnectionManager.Instance.Connection);
                list.ParentCode = code;
                frm = new frmCustomerGroupEntryList();

                frm.List = list;

                //イベントの登録
                this.UpdatedCustomerGroup += new YMyCustomer.CustomerGroupHandler(frm.OnUpdatedCustomerGroup);
                this.DeletedCustomerGroup += new YMyCustomer.CustomerGroupHandler(frm.OnDeletedCustomerGroup);
                this.FormDisposed += new YMyCustomer.DisposedCustomerGroupEntryFormHandler(frm.OnDisposedUpperForm); 
                frm.FormDisposed += new YMyCustomer.DisposedCustomerGroupEntryFormHandler(this.OnDisposedLowerForm);
                
                frm.Show();

                list = null;
                frm = null;
            }
            catch (Exception e)
            {
                if (frm != null)
                {
                    try { frm.Dispose(); }
                    catch { }
                    frm = null;
                }
                if (list != null)
                {
                    try { list.Dispose(); }
                    catch { }
                    list = null;
                }
                YCMessageBox.ShowErrorMessage(e.Message);
            }
        }

        /// <summary>
        /// 顧客グループを追加します。
        /// </summary>
        private void AddCustomerGroup()
        {
            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                this.AddCustomerGroupWithContainer();
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                bool ret = false;
                string groupName = YCInputTools.GetString(out ret, "顧客グループ名入力");

                if (ret)
                {
                    ((YCCustomerGroupEntryList)this.List).Add(groupName, 1);
                }
            }

        }

        /// <summary>
        /// 顧客グループを変更します。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        /// <param name="name">顧客グループ名</param>
        /// <param name="priority">順位</param>
        private void UpdateCustomerGroup(string code, string name, int priority)
        {
            if (code == "") return;

            if (YCLicenseKeys.FrameworkKey != "")
            {
                //
                //YWorks Software Framework3.5用のYWorksキーを取得している場合
                //下記のデータコンテナを使った入力が利用できます。
                //YLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //

                if (this.UpdateCustomerGroupWithContainer(code, name, priority))
                {
                    //顧客グループ変更をアナウンス
                    try {if(this.UpdatedCustomerGroup != null) this.UpdatedCustomerGroup(code); }
                    catch { }
                }
            }
            else
            {
                //
                //YWorks Software Framework3.5用のYWorksキーが無い場合
                //簡易な実装をしていますので、適切に実装し直して下さい。
                //

                bool ret = false;
                string groupName = YCInputTools.GetString(out ret, "顧客グループ名入力", name);

                if (ret)
                {
                    ((YCCustomerGroupEntryList)this.List).Update(code, groupName, 1);

                    //顧客グループ変更をアナウンス
                    try {if (this.UpdatedCustomerGroup != null) this.UpdatedCustomerGroup(code); }
                    catch { }
                }
            }
        }

        /// <summary>
        /// 顧客グループを削除します。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        private void DeleteCustomerGroup(string code)
        {
            if (code == "") return;

            if (YCMessageBox.ShowOKCancelMessage("顧客グループを削除します。", true) == DialogResult.OK)
            {
                ((YCCustomerGroupEntryList)this.List).Delete(code);

                //顧客グループ削除をアナウンス
                try {if (this.DeletedCustomerGroup != null) this.DeletedCustomerGroup(code); }
                catch { }
            }
        }

        #region データコンテナによる入力（YWorks Software Framework3.5用のYWorksキーが必要です。）

        /// <summary>
        /// データコンテナを使って顧客グループ情報を入力
        /// </summary>
        /// <param name="container">顧客グループ登録用コンテナ</param>
        /// <returns>キャンセルならfalseを返す。</returns>
        private bool InputCustomerGroupInformation(YCCustomerGroupEntryContainer container)
        {
            dlgCustomerGroupEntryContainer dlg = null;
            bool ans = false;

            try
            {
                dlg = new dlgCustomerGroupEntryContainer();
                dlg.DataContainer = container;

                if(dlg.ShowDialog(YCApplication.Instance.MdiParent) == DialogResult.OK)
                {
                    ans = true;
                }

                dlg.Dispose();
                dlg = null;

                return ans;

            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }

                throw e;
            }
        }

        /// <summary>
        /// データコンテナを使って顧客グループを追加します。
        /// </summary>
        /// <returns>実行されたならtrue</returns>
        private bool AddCustomerGroupWithContainer()
        {
            YCCustomerGroupEntryContainer container = null;

            try
            {
                container = new YCCustomerGroupEntryContainer();
                container["プライオリティ"] = 1;
                container.IncrementRef();

                bool ret = this.InputCustomerGroupInformation(container);
                if (ret)
                {
                    ((YCCustomerGroupEntryList)this.List).Add(container);
                }

                container.DecrementRef();
                container = null;

                return ret;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客グループを変更します。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        /// <param name="name">顧客グループ名</param>
        /// <param name="priority">順位</param>
        /// <returns>実行されたならtrue</returns>
        private bool UpdateCustomerGroupWithContainer(string code, string name, int priority)
        {
            YCCustomerGroupEntryContainer container = null;

            try
            {
                container = new YCCustomerGroupEntryContainer();
                container["顧客グループコード"] = code;
                container["顧客グループ名"] = name;
                container["プライオリティ"] = priority;
                container.IncrementRef();

                bool ret = this.InputCustomerGroupInformation(container);
                if (ret)
                {
                    ((YCCustomerGroupEntryList)this.List).Update(container);
                }

                container.DecrementRef();
                container = null;

                return ret;
            }
            catch (Exception e)
            {
                if (container != null)
                {
                    container.DecrementRef();
                    try { container.Dispose(); }
                    catch { }
                    container = null;
                }
                throw e;
            }
        }

        #endregion

        #region 顧客グループリストフォーム間のイベント
        /// <summary>
        /// 上位の顧客グループリストで顧客グループ変更時に呼び出されます。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        public void OnUpdatedCustomerGroup(string code)
        {
            if (((YCCustomerGroupEntryList)this.List).ParentCode == code)
            {
                this.SetTitle();
            }
        }

        /// <summary>
        /// 上位の顧客グループリストで顧客グループ削除時に呼び出されます。
        /// </summary>
        /// <param name="code">顧客グループコード</param>
        public void OnDeletedCustomerGroup(string code)
        {
            if (((YCCustomerGroupEntryList)this.List).ParentCode == code)
            {
                this.Close();
            }
        }

        /// <summary>
        /// 上位の顧客グループ登録フォームがDisposeされた時に呼び出されます。
        /// </summary>
        /// <param name="sender">対象オブジェクト</param>
        public void OnDisposedUpperForm(frmCustomerGroupEntryList sender)
        {
            try { this.FormDisposed -= new DisposedCustomerGroupEntryFormHandler(((frmCustomerGroupEntryList)sender).OnDisposedLowerForm); }
            catch { }
        }

        /// <summary>
        /// 下位の顧客グループ登録フォームがDisposeされた時に呼び出されます。
        /// </summary>
        /// <param name="sender">対象オブジェクト</param>
        public void OnDisposedLowerForm(frmCustomerGroupEntryList sender)
        {
            try { this.UpdatedCustomerGroup -= new CustomerGroupHandler(((frmCustomerGroupEntryList)sender).OnUpdatedCustomerGroup); }
            catch { }
            try { this.DeletedCustomerGroup -= new CustomerGroupHandler(((frmCustomerGroupEntryList)sender).OnDeletedCustomerGroup); }
            catch { }
            try { this.FormDisposed -= new DisposedCustomerGroupEntryFormHandler(((frmCustomerGroupEntryList)sender).OnDisposedUpperForm); }
            catch { }
        }

        /// <summary>
        /// 各イベントをクリアします。
        /// </summary>
        private void ClearEvents()
        {
            this.ClearUpdatedCustomerGroup();
            this.ClearDeletedCustomerGroup();
            this.ClearFormDisposed();
        }

        /// <summary>
        /// UpdatedCustomerGroupイベントをクリアします。
        /// </summary>
        private void ClearUpdatedCustomerGroup()
        {

            YMyCustomer.CustomerGroupHandler clone = null;
            try
            {
                if (this.UpdatedCustomerGroup != null)
                {
                    clone = (YMyCustomer.CustomerGroupHandler)this.UpdatedCustomerGroup.Clone();

                    foreach (Delegate d in clone.GetInvocationList())
                    {
                        try { this.UpdatedCustomerGroup -= (YMyCustomer.CustomerGroupHandler)d; }
                        catch { }
                    }
                }
            }
            catch { }
            finally
            {
                clone = null;
            }
        }

        /// <summary>
        /// DeletedCustomerGroupイベントをクリアします。
        /// </summary>
        private void ClearDeletedCustomerGroup()
        {

            YMyCustomer.CustomerGroupHandler clone = null;
            try
            {
                if (this.DeletedCustomerGroup != null)
                {
                    clone = (YMyCustomer.CustomerGroupHandler)this.DeletedCustomerGroup.Clone();

                    foreach (Delegate d in clone.GetInvocationList())
                    {
                        try { this.DeletedCustomerGroup -= (YMyCustomer.CustomerGroupHandler)d; }
                        catch { }
                    }
                }
            }
            catch { }
            finally
            {
                clone = null;
            }
        }

        /// <summary>
        /// FormDisposedイベントをクリアします。
        /// </summary>
        private void ClearFormDisposed()
        {
            YMyCustomer.DisposedCustomerGroupEntryFormHandler clone = null;
            try
            {
                if (this.FormDisposed != null)
                {
                    clone = (YMyCustomer.DisposedCustomerGroupEntryFormHandler)this.FormDisposed.Clone();

                    foreach (Delegate d in clone.GetInvocationList())
                    {
                        try { this.FormDisposed -= (YMyCustomer.DisposedCustomerGroupEntryFormHandler)d; }
                        catch { }
                    }
                }
            }
            catch { }
            finally
            {
                clone = null;
            }
        }

        #endregion

        //イベント処理
        private void frmCustomerGroupEntryList_Load(object sender, EventArgs e)
        {
            try
            {
                this.LoadData();
                this.Centering();

                this.SetTitle();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOpenLowerGroup_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ListGrid.SelectedRows.Count == 0) return;

                string code = YCConvert.ToString(this.ListGrid["顧客グループコード", this.SelectedListGridIndex].Value);
                if (code != "") this.OpenCustomerGroupEntryList(code);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdUpdate_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ListGrid.SelectedRows.Count == 0) return;
                
                string code = YCConvert.ToString(this.ListGrid["顧客グループコード", this.SelectedListGridIndex].Value);
                string name = YCConvert.ToString(this.ListGrid["顧客グループ名", this.SelectedListGridIndex].Value);
                int priority = YCConvert.ToInt(this.ListGrid["プライオリティ", this.SelectedListGridIndex].Value);
                if (code != "") this.UpdateCustomerGroup(code, name, priority);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdAdd_Click(object sender, EventArgs e)
        {
            try
            {
                this.AddCustomerGroup();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDelete_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ListGrid.SelectedRows.Count == 0) return;

                string code = YCConvert.ToString(this.ListGrid["顧客グループコード", this.SelectedListGridIndex].Value);
                if (code != "") this.DeleteCustomerGroup(code);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void dgList_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                if (this.ListGrid.SelectedRows.Count == 0) return;

                //リストのダブルクリック時は下位表示する。
                string code = YCConvert.ToString(this.ListGrid["顧客グループコード", this.SelectedListGridIndex].Value);
                if (code != "") this.OpenCustomerGroupEntryList(code);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void dgList_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            Dictionary<string, string> data = null;

            try
            {
                if (e.Clicks == 1 && e.RowIndex >= 0)
                {
                    //ドラッグアンドドロップ処理（ドラッグ）
                    string customerGroupCode = YCConvert.ToString(dgList["顧客グループコード", this.SelectedListGridIndex].Value);

                    if (customerGroupCode != "")
                    {
                        data = new Dictionary<string, string>();
                        data.Add("顧客グループコード", customerGroupCode);
                        data.Add("顧客グループ名", YCConvert.ToString(dgList["顧客グループ名", this.SelectedListGridIndex].Value));

                        //Dragの開始
                        YCDataGridViewHelper.DoDragDrop(dgList, "顧客グループ", data, DragDropEffects.Copy);
                        //YCDataGridViewHelper.DoDragDrop(dgList, data, DragDropEffects.Copy);

                        data.Clear();
                        data = null;
                    }
                }
            }
            catch (Exception err)
            {
                if (data != null)
                {
                    try { data.Clear(); }
                    catch { }
                    data = null;
                }

                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }
    }
}
