﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客グループメール送信フォーム
    /// </summary>
    public partial class frmCustomerGroupMail : YCMdiChildForm
    {
        private string _customerGroupCode = string.Empty;
        
        public frmCustomerGroupMail()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 顧客グループコード
        /// </summary>
        private string CustomerGroupCode
        {
            get
            {
                return _customerGroupCode;
            }
            set
            {
                _customerGroupCode = value;
            }
        }

        //選択リストの初期化
        protected void SetControlSelections()
        {
            YCControlHelper.FillComboBox(ctlアカウントＩＤ, YCListManager.Instance.Lists["メールアカウント"], false);
            ctlアカウントＩＤ.SelectedIndex = 0;
            
            YCControlHelper.FillComboBox(ctlメール種別コード, YCListManager.Instance.Lists["メール種別"], false);
            ctlメール種別コード.SelectedIndex = 0;
        }

        /// <summary>
        /// 顧客グループの選択
        /// </summary>
        private void SelectCustomerGroup()
        {
            string code = string.Empty;
            string name = string.Empty;

            if (YCApplicationInputTools.SelectCustomerGroupCode(out code, out name, true))
            {
                this.CustomerGroupCode = code;
                this.ctl顧客グループ名.Text = name;
            }
        }

        private void SendMessage()
        {
            YCCustomerGroupMail mail = null;

            try
            {
                mail = new YCCustomerGroupMail(YCConnectionManager.Instance.Connection);

                mail.AccountCode = YCConvert.ToString(ctlアカウントＩＤ.SelectedValue);
                mail.CustomerGroupCode = this.CustomerGroupCode;
                mail.IncludeChildrenCustomerGroup = ctl下位グループフラグ.Checked;
                mail.MailTypeCode = YCConvert.ToString(ctlメール種別コード.SelectedValue);
                mail.MailSubject = ctl件名.Text;
                mail.MailMessage = ctl本文.Text;

                mail.Send();

                mail.Dispose();
                mail = null;
            }
            catch (Exception e)
            {
                if (mail != null)
                {
                    try { mail.Dispose(); }
                    catch { }
                    mail = null;
                }

                throw e;
            }
        }

        //イベント処理
        private void frmCustomerGroupMail_Load(object sender, EventArgs e)
        {
            try
            {
                //
                //YWorks Software Framework キーが設定されている場合のみ
                //処理可能な機能です。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //
                if (YCLicenseKeys.FrameworkKey != "")
                {
                    ctl下位グループフラグ.Visible = true;
                }

                this.SetControlSelections();
                this.Centering();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdSelectCustomerGroup_Click(object sender, EventArgs e)
        {
            try
            {
                this.SelectCustomerGroup();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdSendMessage_Click(object sender, EventArgs e)
        {
            if (YCConvert.ToString(ctlアカウントＩＤ.SelectedValue) == "")
            {
                YCMessageBox.ShowErrorMessage("送信アカウントを指定してください。");
                ctlアカウントＩＤ.Focus();
                return;
            }
            if (this.CustomerGroupCode == "")
            {
                YCMessageBox.ShowErrorMessage("顧客グループを指定してください。");
                cmdSelectCustomerGroup.Focus();
                return;
            }
            if (ctl本文.Text == "")
            {
                YCMessageBox.ShowErrorMessage("本文がありません。");
                ctl本文.Focus();
                return;
            }

            try
            {
                if (YCMessageBox.ShowOKCancelMessage("メッセージを送信します。") == DialogResult.OK)
                {
                    this.SendMessage();

                    YCMessageBox.ShowInformationMessage("メッセージを送信しました。");
                    this.Close();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void ctl顧客グループ名_DragEnter(object sender, DragEventArgs e)
        {
            try
            {
                e.Effect = DragDropEffects.Copy;
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void ctl顧客グループ名_DragDrop(object sender, DragEventArgs e)
        {
            YCDragData data = null;
            try
            {
                //ドロップデータの取得
                data = YCControlHelper.GetDragData(e, "顧客グループ");
                //data = YCControlHelper.GetDragData(e);

                if (data != null)
                {
                    this.CustomerGroupCode = data["顧客グループコード"];
                    ctl顧客グループ名.Text = data["顧客グループ名"];

                    data.Dispose();
                    data = null;
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }
    }
}
