﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客リストフォーム
    /// </summary>
    public partial class frmCustomerList : YCDbListForm
    {
        private string _customerGroupCode = string.Empty;

        public frmCustomerList()
        {
            InitializeComponent();
        }

        /// <summary>
        /// データグリッドビュー
        /// </summary>
        protected override DataGridView ListGrid
        {
            get
            {
                return this.dgList;
            }
        }

        /// <summary>
        /// データグリッドビューの列の作成
        /// </summary>
        protected override void CreateListGridColumns()
        {
            this.AddGridTextColumn("顧客コード");
            this.AddGridTextColumn("顧客名");
            this.AddGridTextColumn("法個人名");
            this.AddGridTextColumn("法人担当者名");
            this.AddGridTextColumn("郵便番号");
            this.AddGridTextColumn("住所");
            this.AddGridTextColumn("取引メモ");
            this.AddGridTextColumn("備考");
        }

        /// <summary>
        /// データグリッドビューの列の書式設定
        /// </summary>
        protected override void FormatListGrid()
        {
            string columnName;

            dgList.ReadOnly = true;
            this.ListGrid.AllowUserToAddRows = false;
            this.ListGrid.AllowUserToDeleteRows = false;
            this.ListGrid.AllowUserToOrderColumns = false;
            this.ListGrid.AllowUserToResizeColumns = true;

            this.ListGrid.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            this.ListGrid.RowTemplate.Height = YCListGridConst.DEFAULT_ROW_HEIGHT;
            this.ListGrid.AlternatingRowsDefaultCellStyle.BackColor = Color.FromName(YCListGridConst.DEFAULT_ALTERNATING_ROWS_BACK_COLOR);
            this.ListGrid.DefaultCellStyle.WrapMode = DataGridViewTriState.False;

            columnName = "顧客コード";
            this.ListGrid.Columns[columnName].Visible = false;

            columnName = "顧客名";
            this.ListGrid.Columns[columnName].Width = 200;

            columnName = "法個人名";
            this.ListGrid.Columns[columnName].Width = 120;
            this.ListGrid.Columns[columnName].HeaderText = "法個人";

            columnName = "法人担当者名";
            this.ListGrid.Columns[columnName].Width = 200;

            columnName = "郵便番号";
            this.ListGrid.Columns[columnName].Width = 120;
            this.ListGrid.Columns[columnName].HeaderText = "〒";

            columnName = "住所";
            this.ListGrid.Columns[columnName].Width = 400;

            columnName = "取引メモ";
            this.ListGrid.Columns[columnName].Width = 400;

            columnName = "備考";
            this.ListGrid.Columns[columnName].Width = 400;
        }

        /// <summary>
        /// 顧客グループコード
        /// </summary>
        private string CustomerGroupCode
        {
            get
            {
                return _customerGroupCode;
            }
            set
            {
                _customerGroupCode = value;
            }
        }

        /// <summary>
        /// リストの表示
        /// </summary>
        private void DrawList()
        {
            ((YCCustomerList)this.List).CustomerGroupCode = this.CustomerGroupCode;
            ((YCCustomerList)this.List).IncludeChildrenCustomerGroup = ctl下位グループフラグ.Checked;

            this.LoadData(true);
            this.ListGrid.Refresh();
            this.ListGrid.Focus();
        }

        //顧客詳細画面を開きます。
        private void OpenCustomer()
        {
            YCCustomer target = null;
            dlgCustomer dlg = null;

            try
            {
                string targetCode = YCConvert.ToString(this.ListGrid["顧客コード", this.SelectedListGridIndex].Value);
                if (targetCode == "") throw new YCException("顧客コードが不正です。");

                target = new YCCustomer(this.List.Connection);
                target.KeyCode = targetCode;

                dlg = new dlgCustomer();
                dlg.Object = target;

                DialogResult ret = dlg.ShowDialog(YCApplication.Instance.MdiParent);
                dlg.Dispose();
                dlg = null;

                if (ret == DialogResult.OK)
                {
                    this.List.RefreshRow(target);
                }

                target.Dispose();
                target = null;
            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }

                if (target != null)
                {
                    try { target.Dispose(); }
                    catch { }
                    target = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客グループの選択
        /// </summary>
        private void SelectCustomerGroup()
        {
            string code = string.Empty;
            string name = string.Empty;

            if (YCApplicationInputTools.SelectCustomerGroupCode(out code, out name, true))
            {
                this.CustomerGroupCode = code;
                this.ctl顧客グループ名.Text = name;
            }
        }

        /// <summary>
        /// CSVファイルの出力
        /// </summary>
        private bool OutputCsv()
        {
            SaveFileDialog dlg = null;
            YCFile file = null;
            bool ret = false;

            try
            {
                dlg = new SaveFileDialog();
                dlg.CheckPathExists = true;
                dlg.FileName = "顧客リスト";
                dlg.AddExtension = true;
                dlg.DefaultExt = "csv";
                dlg.Filter = "CSVファイル|*.csv";
                dlg.InitialDirectory = YCSystemInformationHelper.GetOutputCsvFolder();

                if (dlg.ShowDialog(YCApplication.Instance.MdiParent) == DialogResult.OK)
                {
                    file = new YCFile(dlg.FileName, YCFileMode.OverWrite);
                    YCSystemInformationHelper.SetOutputCsvFolder(YCFileSystem.GetFolderName(dlg.FileName));
                    
                    file.Open();
                    this.List.WriteToFile(file);
                    file.Close();
                   
                    file.Dispose();
                    file = null;

                    ret = true;
                }

                dlg.Dispose();
                dlg = null;

                return ret;
            
            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }

                if (file != null)
                {
                    try { file.Dispose(); }
                    catch { }
                    file = null;
                }

                throw e;
            }
        }

        /// <summary>
        /// 顧客リストの印刷
        /// </summary>
        private void PrintCustomerList()
        {
            if (!YCUser.CurrentUser.CanPrintCustomerList) throw new YCException("印刷する権限がありません。");

            YCPrintDocumentHelper.OpenPreviewForDataGridView(this.ListGrid, PaperKind.A4, true, new Margins(50, 50, 50, 50), new Size(640, 480));
        }

        // イベント
        private void frmCustomerList_Load(object sender, EventArgs e)
        {
            try
            {
                //
                //YWorks Software Framework キーが設定されている場合のみ
                //処理可能な機能です。
                //YCLicenseKeys.FrameworkKeyプロパティを実装して下さい。
                //
                if (YCLicenseKeys.FrameworkKey != "")
                {
                    ctl下位グループフラグ.Visible = true;
                    cmdPrint.Visible = true;
                }

                this.Centering();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDrawList_Click(object sender, EventArgs e)
        {
            try
            {
                this.DrawList();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOpenDetail_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.List.IsLoaded)
                {
                    this.OpenCustomer();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void dgList_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                if (this.List.IsLoaded)
                {
                    this.OpenCustomer();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdSelectCustomerGroup_Click(object sender, EventArgs e)
        {
            try
            {
                this.SelectCustomerGroup();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOutputCsv_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.List.IsLoaded)
                {
                    if (this.OutputCsv())
                    {
                        YCMessageBox.ShowInformationMessage("処理が終了しました。");
                    }
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdPrint_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.List.IsLoaded)
                {
                    this.PrintCustomerList();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void ctl顧客グループ名_DragEnter(object sender, DragEventArgs e)
        {
            try
            {
                e.Effect = DragDropEffects.Copy;
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void ctl顧客グループ名_DragDrop(object sender, DragEventArgs e)
        {
            YCDragData data = null;
            try
            {
                //ドロップデータの取得
                data = YCControlHelper.GetDragData(e, "顧客グループ");
                //data = YCControlHelper.GetDragData(e);

                if (data != null)
                {
                    this.CustomerGroupCode = data["顧客グループコード"];
                    ctl顧客グループ名.Text = data["顧客グループ名"];

                    data.Dispose();
                    data = null;
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }
    }
}
