﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;
using YSystem.Tools;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客名検索フォーム
    /// </summary>
    public partial class frmSearchCustomerNameList : YCSearchNameForm
    {
        public frmSearchCustomerNameList()
        {
            InitializeComponent();
        }

        /// <summary>
        /// データグリッドビュー
        /// </summary>
        protected override DataGridView ListGrid
        {
            get
            {
                return this.dgList;
            }
        }

        /// <summary>
        /// 主キー、名称、名称カナ以外のリスト表示項目の追加
        /// </summary>
        protected override void CreateListGridAdditionalColumns()
        {
            this.AddGridTextColumn("住所");
            this.AddGridCheckBoxColumn("無効フラグ", -1, 0);
        }

        /// <summary>
        /// 名称項目のリスト表示の体裁の初期化
        /// </summary>
        /// <param name="column">名称項目オブジェクト</param>
        protected override void FormatNameGridColumn(DataGridViewColumn column)
        {
            column.Width = 150;
            column.HeaderText = "名称";
        }

        /// <summary>
        /// 名称カナ項目のリスト表示の体裁の初期化
        /// </summary>
        /// <param name="column">名称カナ項目オブジェクト</param>
        protected override void FormatNameKanaGridColumn(DataGridViewColumn column)
        {
            column.Width = 150;
            column.HeaderText = "カナ";
        }

        /// <summary>
        /// 主キー、名称、名称カナ以外のリスト表示項目の体裁の初期化
        /// </summary>
        protected override void FormatAdditionalGridColumns()
        {

            string columnName = "住所";
            this.ListGrid.Columns[columnName].Width = 300;
            this.ListGrid.Columns[columnName].HeaderText = "住所";

            columnName = "無効フラグ";
            this.ListGrid.Columns[columnName].Width = 50;
            this.ListGrid.Columns[columnName].HeaderText = "無効";
        }


        //顧客詳細画面を開きます。
        private void OpenCustomer(bool isAdd)
        {
            YCCustomer target = null;
            dlgCustomer dlg = null;

            try
            {
                target = new YCCustomer(this.List.Connection);

                if (isAdd)
                {
                    //新規登録の場合
                    target.IsNew = true;

                    //すぐに保存する場合は下記実行する。
                    //target.Load();
                    //target.Save();
                }
                else
                {
                    //既存ユーザの場合
                    target.KeyCode = YCConvert.ToString(this.ListGrid["顧客コード", this.SelectedListGridIndex].Value);
                }

                dlg = new dlgCustomer();
                dlg.Object = target;

                DialogResult ret = dlg.ShowDialog(YCApplication.Instance.MdiParent);
                dlg.Dispose();
                dlg = null;

                if (ret == DialogResult.OK && this.QueryIsDone)
                {
                    this.List.RefreshRow(target);
                }

                target.Dispose();
                target = null;
            }
            catch (Exception e)
            {
                if (dlg != null)
                {
                    try { dlg.Dispose(); }
                    catch { }
                    dlg = null;
                }

                if (target != null)
                {
                    try { target.Dispose(); }
                    catch { }
                    target = null;
                }
                throw e;
            }
        }

        /// <summary>
        /// 顧客を削除します。
        /// </summary>
        private void DeleteCustomer()
        {
            string code = YCConvert.ToString(this.ListGrid["顧客コード", this.SelectedListGridIndex].Value);
            if (code == "") return;

            ((YCSearchCustomerNameList)this.List).DeleteCustomer(code);
        }
        
        //イベント処理
        private void cmdAdd_Click(object sender, EventArgs e)
        {
            try
            {
                if (!this.QueryIsDone)
                {
                    YCMessageBox.ShowInformationMessage("既に登録されていないかを検索確認して下さい。");
                    return;
                }

                this.OpenCustomer(true);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOpenDetail_Click(object sender, EventArgs e)
        {
            try
            {
                this.OpenCustomer(false);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdDelete_Click(object sender, EventArgs e)
        {
            try
            {
                if (YCMessageBox.ShowOKCancelMessage("顧客を削除します。", true) == DialogResult.OK)
                {
                    this.DeleteCustomer();
                }
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void dgList_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                this.OpenCustomer(false);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

    }
}
